/*
*  Name:      abstract_application.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Abstract application
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_ABSTRACT_APPLICATION__
#define _MPCL_ABSTRACT_APPLICATION__

#include "text/string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  using text::TString;

  ///  Abstract application class.
  class TAbstractApplication
  {

    private:

      ///  Application name.
      const TString   kyName;

      ///  Application release.
      const TString   kyRelease;
      

    public:

      //
      //  C O N S T R U C T O R S
      //

      /**
      *  Builds a new instance.
      *  @param pkcNAME    Application name.
      *  @param pkcRELEASE Application release.
      */
      TAbstractApplication ( const char* pkcNAME    ,
                             const char* pkcRELEASE ) :
        kyName    (pkcNAME)                           ,
        kyRelease (pkcRELEASE)                        {}

      ///  Destroys the instance.
      virtual ~TAbstractApplication (void) {}

      /**
      *  Initializes application.  In a console application, this means that no
      *  output can be done (other than exceptions or errors).
      */
      virtual void initialize (void) = 0;

      /**
      *  Start the application execution.  In a console application, this means
      *  that any output must begin in this function.
      *  @return Exit status.
      */
      virtual int start (void) = 0;


    public:

      //
      //  S E L E C T O R S
      //

      ///  Get application name.
      TString name (void) const
      {
        return kyName;
      }

      ///  Get application release.
      TString release (void) const
      {
        return kyRelease;
      }

      ///  Get application binary file name.
      virtual TString programFileName (void) const = 0;

  };  // class TAbstractApplication

}  // namespace mpcl


#endif  // not _MPCL_ABSTRACT_APPLICATION__
