/*
*  Name:      exceptions.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Automaton exceptions 
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_AUTOMATON_EXCEPTIONS__
#define _MPCL_AUTOMATON_EXCEPTIONS__

#include "../error_exception.hh"
#include "../exception.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for AUTOMATON.
  namespace automaton
  {

    ///  Exception: parent exception class for AUTOMATON.
    class TException : public virtual mpcl::TException {};

    ///  Exception: DFAML file not found.
    class TFileNotFoundException      :
      public virtual mpcl::TException ,
      public         TErrorException  ,
      public         TException
    {

      private:

        ///  Summary description.
        static const char*   _pkcSynopsis;


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds the instance with a generic and a (if used) specific description
        *  of the exception.
        *  @param rkyRATIONALE  Specific description.
        *  @param rkyFILE_NAME  File name where the exception has thrown.
        *  @param kiLINE_NUMBER Line number of the former file.
        */
        TFileNotFoundException ( const string& rkyRATIONALE  ,
                                 const string& rkyFILE_NAME  ,
                                 const int     kiLINE_NUMBER )                       :
          mpcl::TException (_pkcSynopsis, rkyRATIONALE, rkyFILE_NAME, kiLINE_NUMBER) ,
          TErrorException  ()                                                        ,
          TException       ()                                                        {}

    };  // class TFileNotFoundException

    ///  Exception: integrity failure.
    class TIntegrityException         :
      public virtual mpcl::TException ,
      public         TErrorException  ,
      public         TException
    {

      private:

        ///  Summary description.
        static const char*   _pkcSynopsis;


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds the instance with a generic and a (if used) specific description
        *  of the exception.
        *  @param rkyRATIONALE  Specific description.
        *  @param rkyFILE_NAME  File name where the exception has thrown.
        *  @param kiLINE_NUMBER Line number of the former file.
        */
        TIntegrityException ( const string& rkyRATIONALE  ,
                              const string& rkyFILE_NAME  ,
                              const int     kiLINE_NUMBER )                          :
          mpcl::TException (_pkcSynopsis, rkyRATIONALE, rkyFILE_NAME, kiLINE_NUMBER) ,
          TErrorException  ()                                                        ,
          TException       ()                                                        {}

    };  // class TIntegrityException

    ///  Exception: not a DFAML file.
    class TNotDfamlFileException      :
      public virtual mpcl::TException ,
      public         TErrorException  ,
      public         TException
    {

      private:

        ///  Summary description.
        static const char*   _pkcSynopsis;


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds the instance with a generic and a (if used) specific description
        *  of the exception.
        *  @param rkyRATIONALE  Specific description.
        *  @param rkyFILE_NAME  File name where the exception has thrown.
        *  @param kiLINE_NUMBER Line number of the former file.
        */
        TNotDfamlFileException ( const string& rkyRATIONALE             ,
                                 const string& rkyFILE_NAME  = string() ,
                                 const int     kiLINE_NUMBER = 0        )            :
          mpcl::TException (_pkcSynopsis, rkyRATIONALE, rkyFILE_NAME, kiLINE_NUMBER) ,
          TErrorException  ()                                                        ,
          TException       ()                                                        {}

    };  // class TNotDfamlFileException

    ///  Exception: item (state/event/action) not found.
    class TNotFoundException          :
      public virtual mpcl::TException ,
      public         TErrorException  ,
      public         TException
    {

      private:

        ///  Summary description.
        static const char*   _pkcSynopsis;


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds the instance with a generic and a (if used) specific description
        *  of the exception.
        *  @param rkyRATIONALE  Specific description.
        *  @param rkyFILE_NAME  File name where the exception has thrown.
        *  @param kiLINE_NUMBER Line number of the former file.
        */
        TNotFoundException ( const string& rkyRATIONALE  ,
                             const string& rkyFILE_NAME  ,
                             const int     kiLINE_NUMBER )                           :
          mpcl::TException (_pkcSynopsis, rkyRATIONALE, rkyFILE_NAME, kiLINE_NUMBER) ,
          TErrorException  ()                                                        ,
          TException       ()                                                        {}

    };  // class TNotFoundException

  }  // namespace automaton

}  // namespace mpcl


#endif  // not _MPCL_AUTOMATON_EXCEPTIONS__
