/*
*  Name:         action.hh
*  Authors:      Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Base for action
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 1999-2000  Francisco Vides Fernandez <pvides@dedaloingenieros.com>
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_EVENT_ACTION__
#define _MPCL_EVENT_ACTION__

#include "../text/string.hh"


///  Name-space for MPCL.
namespace mpcl
{
  
  ///  Name-space for EVENT.
  namespace event
  {

    using text::TString;

    ///  Base action class.
    class IAction
    {

      private:

        ///  Action name string.
        TString   yName;


      public:

        //
        // C O N S T R U C T O R S
        //

        /**
        *  Builds a new instance.
        */
        IAction (void) :
          yName ()     {}

        /**
        *  Builds a new instance.
        *  @param rkyNAME Action name.
        */
        IAction (const TString& rkyNAME) :
          yName (rkyNAME)                {}

        ///  Destroys the instance
        virtual ~IAction (void) {}

        /**
        *  Executes the action code.
        *  @return True if action has executed properly.
        */
        virtual bool execute (void) = 0;

        /**
        *  Executes the action code.
        *  @return True if action has executed properly.
        */
        bool operator () (void)
        {
          return execute();
        }

        /**
        *  Executes the reaction due to the fail of action code.
        *  @return Exit code.
        */
        virtual int react (void) = 0;

        /**
        *  Sets action name.
        *  @param rkyNAME New action name.
        */
        void setName (const TString& rkyNAME)
        {
          yName = rkyNAME;
        }


      public:

        //
        // S E L E C T O R S
        //

        /**
        *  Gets action name.
        *  @return Action name.
        */
        const TString& getName (void) const
        {
          return yName;
        }

    };  // class IAction

  }  // namespace event

}  // namespace mpcl


#endif  // not _MPCL_EVENT_ACTION__
