/*
*  Name:      event_handler.hh
*  Authors:   Carlos Jimenez Moreno
*             Rafael Jesus Alcantara Perez
*  Summary:   Base Event Handler
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999       Carlos Jimenez Moreno <carlosj@dedaloingenieros.com>
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/


#ifndef _MPCL_EVENT_EVENT_HANDLER__
#define _MPCL_EVENT_EVENT_HANDLER__

#include <queue>


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for EVENT.
  namespace event
  {

    /**
    *  Base event handler for TStreamableDfa.
    *  The events are sortered by the operator '<' over class TEvent.
    *  @see TStreamableDfa.
    */
    template <typename TEvent>
    class TEventHandler
    {

      protected:

        ///  Event priotity queue type.
        typedef std::priority_queue<TEvent>   TPriorityQueue;

        ///  Event priority queue.
        TPriorityQueue                        tPriorityQueue;


      public:

        //
        // C O N S T R U C T O R S
        //

        ///  Destroys the instance.
        virtual ~TEventHandler (void) {}

        /**
        *  Extracts next event (and removes from the queue).
        *  @return Next event.
        */
        virtual TEvent pop (void)
        {
          TEvent   tEvent = tPriorityQueue.top();

          tPriorityQueue.pop();
          return tEvent;
        }

        /**
        *  Pushes an event into the queue.
        *  @param rktSOURCE_EVENT Pushed event.
        */
        virtual void push (const TEvent& rktSOURCE_EVENT)
        {
          tPriorityQueue.push (rktSOURCE_EVENT);
        }


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Returns true if there is no events queued.
        *  @return True if there is no events queued.
        */
        virtual bool isEmpty (void) const
        {
          return tPriorityQueue.empty();
        }

    };  // TEventHandler

  }  // namespace event

}  // namespace mpcl


#endif // not _MPCL_EVENT_EVENT_HANDLER__
