/*
*  Name:      streamable.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base streamable
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_IO_STREAMABLE__
#define _MPCL_IO_STREAMABLE__

#include <istream>
#include <ostream>
#include <string>
#include "input_streamable.hh"
#include "output_streamable.hh"


///  Name-space for MPCL.
namespace mpcl
{
  
  ///  Name-space for IO.
  namespace io
  {

    ///  Base input-output streamable class.
    template < typename TChar   = char                    ,
               typename TTraits = std::char_traits<TChar> >
    class IStreamable                          :
      public IInputStreamable<TChar, TTraits>  ,
      public IOutputStreamable<TChar, TTraits>
    {

      public:

        ///  Character type.
        typedef
          TChar
          char_type;

        ///  Traits type.
        typedef
          TTraits
          traits_type;

    };
 
  }  // namespace io

}  // namespace mpcl


#endif  // not _MPCL_IO_STREAMABLE__
