/*
*  Name:      thread.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Thread class for POSIX 1003.1c compliant threads
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_POSIX_1_THREAD__
#define _MPCL_SYSTEM_POSIX_1_THREAD__

#include <pthread.h>
#include "../thread.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    /**
    *  Wrapper for POSIX thread and used by operating system class.
    *  @see TOperatingSystem.
    */
    class TThread : public IThread
    {

      private:

        /**
        *  Thread identifier.
        *  Useful for member function join().
        *  @see #join().
        */
        pthread_t   tPosixId;


      protected:

        //
        //  L O C A L   F U N C T I O N S
        //

        /**
        *  Support function for starting threads.
        *  @param pvTHREAD_INSTANCE Pointer to the thread to be started.
        */
        static void* _starter (void* pvTHREAD_INSTANCE)
        {
          ((TThread*) pvTHREAD_INSTANCE)->body();
          return NULL;
        }


      protected:

        //
        //  C O N S T R U C T O R S
        //

        void exit (void)
        {
          pthread_exit (NULL);
        }

        void body (void) = 0;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TThread (void) :
          IThread  ()  ,
          tPosixId (0) {}

        void start (void)
        {
          if ( pthread_create (&tPosixId, NULL, _starter, this) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        void join (void)
        {
          if ( pthread_join (tPosixId, NULL) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

    };  // class TThread

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_POSIX_1_THREAD__
