/*
*  Name:      thread.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base thread
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_THREAD__
#define _MPCL_SYSTEM_THREAD__

#include "defs.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    /**
    *  Thread base class.
    *  @see TBaseOperatingSystem.
    *
    *  \warning All methods must be private or protected to forbids other than
    *           current thread to call own methods. Doing so, it avoids that the
    *           state of the thread could be modified by another external thread
    *           at the same time (race conditions). The exception for this rule
    *           is for communication methods, using for that, mutexes or so.
    */
    class IThread
    {

      protected:

        /**
        *  Stops the thread.
        *  This member function musts be called only by the current thread.
        */
        virtual void exit (void) = 0;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Destroys the instance.
        virtual ~IThread (void) {}

        ///  Starts the thread.
        virtual void start (void) = 0;

        ///  From other thread, waits til this thread has finished.
        virtual void join (void) = 0;


      protected:

        //
        //  C O N S T R U C T O R S
        //

        ///  Main body of the thread.
        virtual void body (void) = 0;

    };  // class IThread

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_THREAD__
