/*
*  Name:      clause_tag_string.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Clause tag string
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_CODEGEN_CLAUSE_TAG_STRING__
#define _MPCL_TEXT_CODEGEN_CLAUSE_TAG_STRING__

#include "../../io/streamable.hh"
#include "../string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for CODEGEN.
    namespace codegen
    {

      using io::IStreamable;

      ///  Forwarded declaration.
      class TClauseTagString;

      /**
      *  Reads the instance \a ryTARGET_CLAUSE_TAG from stream \a rtSOURCE_ISTREAM.
      *  This operator is needed cause father operators.
      */
      template <typename TChar, typename TTraits>
      std::basic_istream<TChar, TTraits>&
      operator >> ( std::basic_istream<TChar, TTraits>& rtSOURCE_ISTREAM    ,
                    TClauseTagString&                   ryTARGET_CLAUSE_TAG );

      /**
      *  Writes the instance \a rkySOURCE_CLAUSE_TAG onto stream \a rtTARGET_STREAM.
      *  This operator is needed cause father operators.
      */
      template <typename TChar, typename TTraits>
      std::basic_ostream<TChar, TTraits>&
      operator << ( std::basic_ostream<TChar, TTraits>& rtSOURCE_OSTREAM     ,
                    const TClauseTagString&             rkySOURCE_CLAUSE_TAG );

      /**
      *  Clause tag string for pattern based code generator.
      *  @see TPatternBasedCodeGenerator.
      */
      class TClauseTagString :
        public TString       ,
        public IStreamable<>
      {

        public:

          ///  Character type.
          typedef
            IStreamable<>::char_type
            char_type;

          ///  Traits type.
          typedef
            IStreamable<>::traits_type
            traits_type;


        private:

          /**
          *  Target variable.
          *  It can be used externally to maintain a map with variables.
          */
          TString   yTarget;

          ///  True if there is no parameter-existence sensitiveness.
          bool      gCheckParameters;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Build a new instance.
          TClauseTagString (void)   :
            TString          ()     ,
            IStreamable<>    ()     ,
            yTarget          ()     ,
            gCheckParameters (true) {}

          /**
          *  Build a new instance from \a pkcSOURCE_TEXT.
          *  @param pkcSOURCE_TEXT Source text.
          */
          TClauseTagString (const char* pkcSOURCE_TEXT) :
            TString          (pkcSOURCE_TEXT)           ,
            IStreamable<>    ()                         ,
            yTarget          ()                         ,
            gCheckParameters (true)                     {}

          /**
          *  Build a new instance from \a rkySOURCE_CLAUSE_TAG.
          *  @param rkySOURCE_CLAUSE_TAG Source clause tag.
          */
          TClauseTagString (const TClauseTagString& rkySOURCE_CLAUSE_TAG) :
            TString          (rkySOURCE_CLAUSE_TAG)                       ,
            IStreamable<>    ()                                           ,
            yTarget          (rkySOURCE_CLAUSE_TAG.yTarget)               ,
            gCheckParameters (rkySOURCE_CLAUSE_TAG.gCheckParameters)      {}

          /**
          *  Copies the \a rkySOURCE_CLAUSE_TAG onto this instance.
          *  @param rkySOURCE_CLAUSE_TAG Source clause tag.
          */
          void overloadFrom (const TClauseTagString& rkySOURCE_CLAUSE_TAG);

          ///  Reads instance from stream \a rsSOURCE_ISTREAM.
          void read (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM);


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns true if clause has target.
          *  @return True if clause has target.
          */
          bool hasTarget (void) const
          {
            return !target().empty();
          }

          bool mustCheckParameters (void) const;

          const TString& name (void) const;

          const TString& target (void) const;

          ///  Writes instance onto stream \a rsTARGET_OSTREAM.
          void write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const;


        public:

          //
          //  F R I E N D   F U N C T I O N S
          //

          friend std::basic_istream<value_type, traits_type>&
          operator >> <value_type, traits_type>
          (
            std::basic_istream<value_type, traits_type>& rtSOURCE_ISTREAM    ,
            TClauseTagString&                            ryTARGET_CLAUSE_TAG
          );

          friend std::basic_ostream<value_type, traits_type>&
          operator << <value_type, traits_type>
          (
            std::basic_ostream<value_type, traits_type>& rtSOURCE_OSTREAM     ,
            const TClauseTagString&                      rkySOURCE_CLAUSE_TAG
          );

      };  // class TClauseTagString

      //
      //  F R I E N D   F U N C T I O N S
      //

      template <typename TChar, typename TTraits>
      std::basic_istream<TChar, TTraits>&
      operator >> ( std::basic_istream<TChar, TTraits>& rtSOURCE_ISTREAM    ,
                    TClauseTagString&                   ryTARGET_CLAUSE_TAG )
      {
        ryTARGET_CLAUSE_TAG.read (rtSOURCE_ISTREAM);
        return rtSOURCE_ISTREAM;
      }

      template <typename TChar, typename TTraits>
      std::basic_ostream<TChar, TTraits>&
      operator << ( std::basic_ostream<TChar, TTraits>& rtSOURCE_OSTREAM     ,
                    const TClauseTagString&             rkySOURCE_CLAUSE_TAG )
      {
        rkySOURCE_CLAUSE_TAG.write (rtSOURCE_OSTREAM);
        return rtSOURCE_OSTREAM;
      }

    }  // namespace codegen

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_CODEGEN_CLAUSE_TAG_STRING__
