/*
*  Name:      instruction_body.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Instruction body
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_CODEGEN_INSTRUCTION_BODY__
#define _MPCL_TEXT_CODEGEN_INSTRUCTION_BODY__

#include "../../io/streamable.hh"
#include "../../io/streamable_map.hh"
#include "../../util/collection/map.hh"
#include "../string.hh"
#include "clause_body_string.hh"
#include "clause_tag_string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for CODEGEN.
    namespace codegen
    {

      using io::IStreamable;
      using io::TStreamableMap;
      using std::basic_istream;
      using std::basic_ostream;

      /**
      *  Instruction body for pattern based code generator.
      *  @see TPatternBasedCodeGenerator.
      */
      class TInstructionBody : public IStreamable<>
      {

          //
          //  F R I E N D   C L A S S E S
          //

          friend class TPatternBasedCodeGenerator;


        private:

          ///  Map with clauses type.
          typedef
            TStreamableMap<TClauseTagString, TClauseBodyString>
            TClauseMap;


        private:

          ///  Map with clauses.
          TClauseMap   tClauseMap;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TInstructionBody (void) :
            IStreamable<> ()      ,
            tClauseMap    ()      {}

          /**
          *  Copies from instance \a rktSOURCE_INSTRUCTION_BODY.
          *  @param rktSOURCE_INSTRUCTION_BODY Source instance.
          *  @return This instance.
          */
          TInstructionBody& operator = (const TInstructionBody& rktSOURCE_INSTRUCTION_BODY);


        protected:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Loads the instance from stream \a rtSOURCE_ISTREAM.
          *  @param rtSOURCE_ISTREAM Source stream.
          */
          void read (basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM);


        public:

          //
          //  S E L E C T O R S
          //

          bool existsClause (const char* pkcCLAUSE_NAME) const;

          const TClauseTagString& clauseTag (const char* pkcSOURCE_CLAUSE_TAG) const;

          const TClauseBodyString& clauseBody (const char* pkcSOURCE_CLAUSE_TAG) const;

          TMap<TString, const TClauseBodyString*> targetedVariablesForClause (const char* pkcSOURCE_CLAUSE_TAG) const;

          const TClauseBodyString& operator [] (const char* pkcSOURCE_CLAUSE_TAG) const;

          /**
          *  Returns true if both instances are equal.
          *  @param rktSOURCE_INSTRUCTION_BODY Source instance.
          *  @return True if both instances are equally named.
          */
          bool operator == (const TInstructionBody& rktSOURCE_INSTRUCTION_BODY) const;


        protected:

          //
          //  S E L E C T O R S
          //

          /**
          *  Writes the instance onto stream \a rtTARGET_OSTREAM.
          *  @param rtTARGET_STREAM Target stream.
          */
          void write (basic_ostream<char_type, traits_type>& rtTARGET_STREAM) const;

      };  // class TInstructionBody

    }  // namespace codegen

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_CODEGEN_INSTRUCTION_BODY__
