/*
*  Name:      anchor.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML anchor
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_ANCHOR__
#define _MPCL_TEXT_HTML_ANCHOR__

#include "../../net/url_encoder.hh"
#include "container.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      ///  Forward declaration.
      class TAnchor;

      ///  Smart pointer to TAnchor.
      typedef
        TSmartPointer<TAnchor>
        QTAnchor;

      ///  Smart pointer to const TAnchor.
      typedef
        TSmartPointer<const TAnchor>
        QKTAnchor;

      /**
      *  HTML anchor.
      *
      *  Attributes added:
      *
      *    "href"  ::= A.HREF.
      *    "name"  ::= A.NAME.
      *    "title" ::= A.TITLE.
      */
      class TAnchor : public TContainer
      {

        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance.
          *  @param pkcHREF   HREF field.
          *  @param pkcNAME   NAME field.
          *  @param pkcTITLE  TITLE field.
          *  @param pkcTARGET TARGET field.
          */
          TAnchor ( const char* pkcHREF        ,
                    const char* pkcNAME        ,
                    const char* pkcTITLE       ,
                    const char* pkcTARGET = "" ) :
            TContainer (pkcNAME)
          {
            tAttributeMap ["class-identifier"] = "anchor";
            tAttributeMap.bind ("name",   pkcNAME);
            tAttributeMap.bind ("href",   pkcHREF);
            tAttributeMap.bind ("title",  pkcTITLE);
            tAttributeMap.bind ("target", pkcTARGET);
            gIsQualified = true;
          }

          /**
          *  Builds a new instance.
          *  @param pkcHREF   HREF field.
          *  @param pkcNAME   NAME field.
          *  @param pkcTITLE  TITLE field.
          *  @param rkqtTAG   Tag to embed in.
          *  @param pkcTARGET TARGET field.
          */
          TAnchor ( const char*  pkcHREF        ,
                    const char*  pkcNAME        ,
                    const char*  pkcTITLE       ,
                    const QTTag& rkqtTAG        ,
                    const char*  pkcTARGET = "" ) :
            TContainer (pkcNAME)
          {
            tAttributeMap ["class-identifier"] = "anchor";
            tAttributeMap.bind ("name",   pkcNAME);
            tAttributeMap.bind ("href",   pkcHREF);
            tAttributeMap.bind ("title",  pkcTITLE);
            tAttributeMap.bind ("target", pkcTARGET);
            insert (rkqtTAG);
            gIsQualified = true;
          }

          /**
          *  Builds a new instance.
          *  @param pkcHREF                 HREF field.
          *  @param pkcNAME                 NAME field.
          *  @param pkcTITLE                TITLE field.
          *  @param rkqtTAG                 Tag to embed in.
          *  @param rktSTRING_TO_STRING_MAP Map with parameters.
          *  @param pkcTARGET               TARGET field.
          */
          TAnchor ( const char*               pkcHREF                 ,
                    const char*               pkcNAME                 ,
                    const char*               pkcTITLE                ,
                    const QTTag&              rkqtTAG                 ,
                    const TStringToStringMap& rktSTRING_TO_STRING_MAP ,
                    const char*               pkcTARGET = ""          ) :
            TContainer (pkcNAME)
          {
            tAttributeMap ["class-identifier"] = "anchor";
            tAttributeMap.bind ("name",   pkcNAME);
            tAttributeMap.bind ("href",   pkcHREF);
            tAttributeMap.bind ("title",  pkcTITLE);
            tAttributeMap.bind ("target", pkcTARGET);
            insert (rkqtTAG);
            gIsQualified            = true;
            tAttributeMap ["href"] += "?";
            tAttributeMap ["href"] += net::TUrlEncoder::_encode (rktSTRING_TO_STRING_MAP, false);
          }

      };  // class TAnchor

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_ANCHOR__
