/*
*  Name:      body.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML body
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_BODY__
#define _MPCL_TEXT_HTML_BODY__

#include "container.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      ///  Forward declaration.
      class TBody;

      ///  Smart pointer to TBody.
      typedef
        TSmartPointer<TBody>
        QTBody;

      ///  Smart pointer to const TBody.
      typedef
        TSmartPointer<const TBody>
        QKTBody;

      /**
      *  HTML body.
      *
      *  Attributes added:
      *
      *    "background" ::= BODY.BACKGROUND.
      *    "bgcolor"    ::= BODY.BGCOLOR.
      *    "text"       ::= BODY.
      *    "link"       ::= BODY.LINK.
      *    "vlink"      ::= BODY.VLINK.
      *    "alink"      ::= BODY.ALINK.
      */
      class TBody : public TContainer
      {

        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance (there can be only one body).
          TBody (void) : TContainer (pkcDefaultBodyId)
          {
            tAttributeMap ["class-identifier"] = "body";
            gIsQualified                       = true;
            tAttributeMap.bind ("background", "");
            tAttributeMap.bind ("bgcolor",    "#BBBBBB");
            tAttributeMap.bind ("text",       "#101010");
            tAttributeMap.bind ("link",       "#1010FF");
            tAttributeMap.bind ("vlink",      "#1010BB");
            tAttributeMap.bind ("alink",      "#FF1010");
          }

      };  // class TBody

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_BODY__
