/*
*  Name:      defs.hh
*  Author:    Angel Jimenez Jimenez
*             Rafael Jesus Alcantara Perez
*  Summary:   HTML definitions
*  Date:      $Date: 2005-03-08 00:21:07 +0100 (mar, 08 mar 2005) $
*  Revision:  $Revision: 10851 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_DEFS__
#define _MPCL_TEXT_HTML_DEFS__

#include <functional>
#include "exceptions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    /**
    *  Name-space for HTML.
    *  This name-space makes the variables contained not to collide
    *  with any others in MPCL name-space.
    */
    namespace html
    {

      //
      //  E X P O R T E D   V A R I A B L E S
      //

      ///  Default identifier for THtmlBody.
      extern const char*   pkcDefaultBodyId;

      ///  Default identifier for THtmlFrameset.
      extern const char*   pkcDefaultFramesetId;

      ///  Default identifier for THtmlHead.
      extern const char*   pkcDefaultHeadId;

      ///  Default identifier for THtmlTitle.
      extern const char*   pkcDefaultTitleId;

      ///  MIME type for Cascade Style Sheets.
      extern const char*   pkcTypeTextCss;

      ///  Separator character between two identifiers.
      extern const char   kcIdentifierSeparator;

      ///  Separator character between identifier and value.
      extern const char   kcIdentifierValueSeparator;


      //
      //  E X P O R T E D   F U N C T I O N   O B J E C T S
      //

      /**
      *  Unary function object  to  search  for an instance  in  a map using member
      *  method \a identifier().
      */
      template <typename TTargetIdentifier, typename TSourceItem>
      struct MMatchPairIdentifier : public std::unary_function<TSourceItem, bool>
      {

        public:

          ///  Identifier to search for.
          TTargetIdentifier   tTargetIdentifier;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance.
          */
          MMatchPairIdentifier (void) :
            tTargetIdentifier()       {}

          /**
          *  Builds a new instance for searching \a rktTARGET_IDENTIFIER.
          *  @param rktTARGET_IDENTIFIER Identifier to search for.
          */
          MMatchPairIdentifier (const TTargetIdentifier& rktTARGET_IDENTIFIER) :
            tTargetIdentifier (rktTARGET_IDENTIFIER)                           {}

          /**
          *  Sets target for searching.
          *  @param rktTARGET_IDENTIFIER Identifier to search for.
          */
          void setTarget (const TTargetIdentifier& rktTARGET_IDENTIFIER)
          {
            tTargetIdentifier = rktTARGET_IDENTIFIER;
          }

          /**
          *  Writes identifier of \a rktSOURCE_ITEM.
          *  @param rktSOURCE_ITEM Identifier to append.
          */
          bool operator () (const TSourceItem& rktSOURCE_ITEM)
          {
            return ( rktSOURCE_ITEM.second->identifier() == tTargetIdentifier );
          }

      };  // struct MMatchPairIdentifier

      /**
      *  Unary  function  object  to write an HTML tag  instance  identifier  to  a
      *  stream.
      */
      template <typename TTargetItem, typename TSourceItem>
      struct MWriteIdentifierTo : public std::unary_function<TTargetItem, void>
      {

        public:

          ///  Target item for appending.
          TTargetItem&   rtTargetItem;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance  for  appending \a rktTARGET_ITEM  to a provided
          *  \a TTargetItem instance (that must be a pointer).
          *  @param rtTARGET_ITEM Item to append.
          */
          MWriteIdentifierTo (TTargetItem& rtTARGET_ITEM) :
            rtTargetItem (rtTARGET_ITEM)                  {}

          /**
          *  Writes identifier of \a rtSOURCE_ITEM.
          *  @param rtSOURCE_ITEM Item to append.
          */
          void operator () (TSourceItem& rtSOURCE_ITEM)
          {
            rtTargetItem << rtSOURCE_ITEM->identifier() << std::endl;
          }

          /**
          *  Writes identifier of \a rktSOURCE_ITEM.
          *  @param rktSOURCE_ITEM Item to append.
          */
          void operator () (const TSourceItem& rktSOURCE_ITEM)
          {
            rtTargetItem << rktSOURCE_ITEM->identifier() << std::endl;
          }

      };  // struct MWriteIdentifierTo

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_DEFS__
