/*
*  Name:      frameset.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML frameset
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_FRAMESET__
#define _MPCL_TEXT_HTML_FRAMESET__

#include "container.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      ///  Forward declaration.
      class TFrameset;

      ///  Smart pointer to TFrameset.
      typedef
        TSmartPointer<TFrameset>
        QTFrameset;

      ///  Smart pointer to const TFrameset.
      typedef
        TSmartPointer<const TFrameset>
        QKTFrameset;

      ///  HTML frameset.
      class TFrameset : public TContainer
      {

        public:

          //
          //  C O N S T R U C T O R S
          //

          TFrameset ( const char* pkcROWS                  ,
                      const char* pkcCOLS                  ,
                      const char* pkcBORDER                ,
                      const char* pkcBORDERCOLOR  = "gray" ,
                      const char* pkcFRAMEBORDER  = "NO"   ,
                      const char* pkcFRAMESPACING = "0"    ) :
            TContainer (pkcDefaultFramesetId)
          {
            tAttributeMap ["class-identifier"] = pkcDefaultFramesetId;
            tAttributeMap.bind ("cols",         pkcCOLS);
            tAttributeMap.bind ("border",       pkcBORDER);
            tAttributeMap.bind ("bordercolor",  pkcBORDERCOLOR);
            tAttributeMap.bind ("frameborder",  pkcFRAMEBORDER);
            tAttributeMap.bind ("framespacing", pkcFRAMESPACING);
            tAttributeMap.bind ("rows",         pkcROWS);
          }

      };  // class TFrameset

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_FRAMESET__
