/*
*  Name:      layout_table.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML layout table
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_LAYOUT_TABLE__
#define _MPCL_TEXT_HTML_LAYOUT_TABLE__

#include "table.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      ///  Forward declaration.
      class TLayoutTable;

      ///  Smart pointer to TLayoutTable.
      typedef
        TSmartPointer<TLayoutTable>
        QTLayoutTable;

      ///  Smart pointer to const TLayoutTable.
      typedef
        TSmartPointer<const TLayoutTable>
        QKTLayoutTable;

      /**
      *  HTML layout table.
      *
      *  All HTML tags inserted  in the  table cells belonging to this table,  will
      *  have  a  canonical identifier  that defines the way trough  the  table  to
      *  reach the tag  contained  in the table-cell.  In this table, the rows, and
      *  cells have no way to be accessed through indexing member methods. Only the
      *  tags contained in every cell can be accessed in that way.
      */
      class TLayoutTable : public TTable
      {

        public:

          //
          //  C O N S T R U C T O R S
          //

          TLayoutTable ( const char* pkcIDENTIFIER         ,
                         const char* pkcCAPTION = ""       ,
                         const char* pkcALIGN   = "center" ) :
            TTable (pkcIDENTIFIER, pkcCAPTION, pkcALIGN)     {}

          /**
          *  Tries to  find a tag named  \a rkyIDENTIFIER  from the tags stored (if
          *  any) in all cells, and in all rows in this instance. If the tag is not
          *  found then a new \a TTableRow with a new \a TTableCell with a
          *  new \a TTag (named as \a rkyIDENTIFIER) is inserted.
          *  @param rkyIDENTIFIER Source tag identifier.
          *  @return Returns that row.
          */
          QTTag findOrInsertTag (const TString& rkyIDENTIFIER);


        public:

          //
          //  S E L E C T O R S
          //

          QTTag findTag (const TString& rkyIDENTIFIER) const;

      };  // class TLayoutTable

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_LAYOUT_TABLE__
