/*
*  Name:      page.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML page
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_HTML_PAGE__
#define _MPCL_TEXT_HTML_PAGE__

#include "base_page.hh"
#include "body.hh"
#include "head.hh"
#include "title.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for HTML.
    namespace html
    {

      ///  Forward declaration.
      class TPage;

      ///  Smart pointer to TPage.
      typedef
        TSmartPointer<TPage>
        QTPage;

      ///  Smart pointer to const TPage.
      typedef
        TSmartPointer<const TPage>
        QKTPage;

      ///  HTML page.
      class TPage : public TBasePage
      {

        protected:

          ///  Body of the page.
          QTBody   qtBody;

          ///  Head of the page.
          QTHead   qtHead;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance and inserts the head and the body.
          *  @param pkcIDENTIFIER Tag identifier.
          */
          TPage (const char* pkcIDENTIFIER) :
            TBasePage (pkcIDENTIFIER)       ,
            qtBody    (new TBody())         ,
            qtHead    (new THead())
          {
            tAttributeMap ["class-identifier"] = "page";
            insert (QTTag (qtHead));
            insert (QTTag (qtBody));
          }

          /**
          *  Builds a new instance and inserts the head and the body.  Also inserts
          *  a title (\a pkcTITLE) into the head.
          *  @param pkcIDENTIFIER Tag identifier.
          *  @param pkcTITLE      Title of the page.
          */
          TPage (const char* pkcIDENTIFIER, const char* pkcTITLE) :
            TBasePage (pkcIDENTIFIER)                             ,
            qtBody    (new TBody())                               ,
            qtHead    (new THead())
          {
            tAttributeMap ["class-identifier"] = "page";
            insert (QTTag (qtHead));
            insert (QTTag (qtBody));
            qtHead->insert (QTTag (new TTitle (pkcTITLE)));
          }

      };  // class TPage

    }  // namespace html

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_HTML_PAGE__
