/*
*  Name:         local_date.hh
*  Author:       Francisco Rodrigo Escobedo Robles
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Local date class
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 2000-2001  Francisco Rodrigo Escobedo Robles <frer@pepix.net>
*  Copyright (C) 2002       Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_LOCAL_DATE__
#define _MPCL_TEXT_LOCAL_DATE__

#include "date.hh"
#include "string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

     /**
     *  Local date abstract data type.
     */
     class TLocalDate : public IDate
     {
   
       public:
   
         //
         //  C O N S T R U C T O R S
         //
   
         /**
         *  Builds a new instance.
         *  @param kiYEAR    Year.
         *  @param kiMONTH   Month   [1-12].
         *  @param kiDAY     Day     [1-31].
         *  @param kiHOUR    Hour    [0-23] (0).
         *  @param kiMINUTES Minutes [0-59] (0).
         *  @param kiSECONDS Seconds [0-59] (0).
         */
         TLocalDate ( const int kiYEAR        ,
                      const int kiMONTH       ,
                      const int kiDAY         ,
                      const int kiHOUR    = 0 ,
                      const int kiMINUTES = 0 ,
                      const int kiSECONDS = 0 ) : 
           IDate ( kiYEAR    ,
                   kiMONTH   ,
                   kiDAY     ,
                   kiHOUR    ,
                   kiMINUTES ,
                   kiSECONDS )                  {}
   
         
         /**
         *  Builds a new instance.
         *  @param keDATE_TYPE Type of the date to create.
         *  @see #EDateType.
         */
         TLocalDate (const EDateType keDATE_TYPE = eNow)
           throw (TConstraintException) :
           IDate (keDATE_TYPE)          {}
   
   
       public:
   
         //
         //  S E L E C T O R S
         //
   
         /**
         *  Gets the current date in an specified strftime-like format.
         *  @see std::strftime().
         */
         TString get (const char* pkcDATE_FORMAT) const;
   
     };  // class TLocalDate
   
  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_LOCAL_DATE__
