/*
*  Name:      string.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Tools for TString
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_STRING__
#define _MPCL_TEXT_STRING__

#include <algorithm>
#include "basic_string.hh"


///  Name-space for MPCL.
namespace mpcl
{
  
  ///  Name-space for TEXT.
  namespace text
  {

    //
    //  E X P O R T E D   T Y P E S
    //

    ///  Extended string specialization.
    typedef TBasicString<char>   TString;


    //
    //  E X P O R T E D   F U N C T I O N S
    //

    /**
    *  Returns \a pkcTRUE if \a gTRUTH is true, else returns \a pkcFALSE.
    *  @param gTRUTH   Value to test.
    *  @param pkcTRUE  String for true value.
    *  @param pkcFALSE String for false value.
    *  @return if \a gTRUTH then \a pkcTRUE, else \a pkcFALSE.
    */
    const char* BoolToString ( bool        gTRUTH             ,
                               const char* pkcTRUE  = "true"  ,
                               const char* pkcFALSE = "false" );

    /**
    *  The behavior of this function is like std::printf() but returning a
    *  string instance.
    *  @param pkcFORMAT Format string (must not be NULL).
    *  @param ...       Arguments.
    *  @return Formated string.
    *  @see std::printf().
    */
    TString Format (const char* pkcFORMAT...);

    /**
    *  Returns a copy of \a rkySOURCE but with all characters changed to
    *  lowercase (using std::tolower()).
    *  @param rkySOURCE Source string.
    *  @return Lowercase string.
    *  @see std::tolower().
    */
    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>
    Lowercase (const std::basic_string<TChar, TTraits, TAllocator>& rkySOURCE)
    {

      TBasicString<TChar, TTraits, TAllocator>   yTarget = rkySOURCE;

      std::transform (yTarget.begin(), yTarget.end(), yTarget.begin(), tolower);
      return yTarget;

    }  // Lowercase()

    /**
    *  Returns a copy of \a rkySOURCE but with all characters changed to
    *  uppercase (using std::toupper()).
    *  @param rkySOURCE Source string.
    *  @return Uppercase string.
    *  @see std::toupper().
    */
    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>
    Uppercase (const std::basic_string<TChar, TTraits, TAllocator>& rkySOURCE)
    {

      TBasicString<TChar, TTraits, TAllocator>   yTarget = rkySOURCE;

      std::transform (yTarget.begin(), yTarget.end(), yTarget.begin(), toupper);
      return yTarget;

    }  // Uppercase()

    /**
    *  Stores in a container, the elements in [first, last) separated
    *  by tSEPARATOR.
    *  @param tBEGIN_ITER  Iterator start.
    *  @param tEND_ITER    Iterator end.
    *  @param tSEPARATOR   Separator.
    *  @param tTARGET_ITER Target iterator.
    *  @return The target iterator.
    */
    template <typename TInputIter, typename TChar, typename TOutputIter>
    inline TOutputIter Split ( TInputIter  tBEGIN_ITER  ,
                               TInputIter  tEND_ITER    ,
                               TChar       tSEPARATOR   ,
                               TOutputIter tTARGET_ITER )
    {

      TInputIter   tSeparatorIter;
      TString      ySubstring;

      while ( tEND_ITER != (tSeparatorIter = std::find (tBEGIN_ITER, tEND_ITER, tSEPARATOR)) )
      {
        ySubstring = TString (tBEGIN_ITER, tSeparatorIter);
        if ( ySubstring.length() )
        {
          *tTARGET_ITER = ySubstring;
          ++tTARGET_ITER;
        }
        tBEGIN_ITER = tSeparatorIter + 1;
      }

      //
      //  Add last path item if any.
      //
      if ( tBEGIN_ITER != tEND_ITER )
      {
        ySubstring    = TString (tBEGIN_ITER, tEND_ITER);
        *tTARGET_ITER = ySubstring;
      }
      return tTARGET_ITER;

    }  // Split()

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_STRING__
