/*
*  Name:      sort_algorithm.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base sort algorithm
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_STRATEGY_SORT_ALGORITHM__
#define _MPCL_UTIL_STRATEGY_SORT_ALGORITHM__


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for UTIL.
  namespace util
  {

    ///  Name-space for STRATEGY.
    namespace strategy
    {

      /**
      *  Base sort algorithm.
      *
      *  Sort algorithms from STL are template functions, but these lets the client
      *  to extends these algorithms using OOP techniques.
      *
      *  Requires:
      *
      *    TSequence::const_iterator
      *    TSequence::begin()
      *    TSequence::end()
      *    TSequence::push_back()
      *    TSequence::value_type
      */
      template <typename TSequence>
      class ISortAlgorithm
      {

        public:

          ///  Constant iterator for sequences.
          typedef
            typename TSequence::const_iterator
            const_iterator;

          ///  Iterator for sequences.
          typedef
            typename TSequence::iterator
            iterator;

          ///  Value type for sequences.
          typedef
            typename TSequence::value_type
            value_type;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Applies the sort algorithm over the sequence of
          *  items \a rtSOURCE_SEQUENCE.
          *  @param rtSOURCE_SEQUENCE Source sequence.
          */
          virtual void execute (TSequence& rtSOURCE_SEQUENCE) = 0;


        protected:

          //
          //  S E L E C T O R S
          //

          /**
          *  Swaps values of items \a rtFIRST_ITEM and \a rtSECOND_ITEM.
          *  @param rtFIRST_ITEM  Target item.
          *  @param rtSECOND_ITEM Target item.
          */
          void swap (value_type& rtFIRST_ITEM, value_type& rtSECOND_ITEM)
          {
            value_type   tTempItem (rtFIRST_ITEM);

            rtFIRST_ITEM  = rtSECOND_ITEM;
            rtSECOND_ITEM = tTempItem;
          }

      };  // class ISortAlgorithm

    }  // namespace strategy

  }  // namespace util

}  // namespace mpcl


#endif  // not _MPCL_UTIL_STRATEGY_SORT_ALGORITHM__
