/*
*  Name:      exception.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   MPCL base exception
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <mpcl/exception.hh>


//
//  S E L E C T O R S
//

const std::string mpcl::TException::
what (void) const
{

  using std::string;

  string   yDescription;
  
  if ( yFileName.size() )
  {
    //
    //  If there is a file-name then appends it.
    //
    yDescription.append (yFileName);
    yDescription.append (1, ':');

    //
    //  Appends the line-number converted to an ASCII character representation.
    //
    {
      string::size_type   zInsertionPoint = yDescription.length();
      size_t              zLineNumberAux  = iLineNumber;

      while ( zLineNumberAux >= 10 )
      {
        yDescription.insert (zInsertionPoint, 1, '0' + (zLineNumberAux % 10));
        zLineNumberAux /= 10;
      }
      yDescription.insert (zInsertionPoint, 1, '0' + zLineNumberAux);
    }

    //
    //  Appends last part of the header of the exception.
    //
    yDescription.append (1, ':');
  }

  //
  //  Appends the general description and the specific description, if any.
  //
  yDescription.append (ySynopsis);
  if ( yRationale.size() )
  {
    yDescription.append ("; ");
    yDescription.append (yRationale);
  }
  return yDescription;

}  // what()
