/*
*  Name:      cgi_cookie.cc
*  Author:    Francisco Rodrigo Escobedo Robles
*  Summary:   HTTP Cookie handling
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2000  Francisco Rodrigo Escobedo Robles <frer@pepix.net>
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/net/cgi/cookie.hh>


//
//  C O N S T R U C T O R S
//

void mpcl::net::cgi::TCookie::
read (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM)
{

  char   cTmpChar;

  yName  = "";
  yValue = "";

  //
  //  Skips blank space.
  //
  while ( ( (cTmpChar = rtSOURCE_ISTREAM.get()) == ' ' ) && rtSOURCE_ISTREAM.good() );

  //
  //  Reads the name.
  //
  if ( cTmpChar != EOF )
  {
    do
    {
      yName += cTmpChar;
    }
    while ( ( (cTmpChar = rtSOURCE_ISTREAM.get()) != '=' ) && rtSOURCE_ISTREAM.good() );
    
    //
    //  Reads the value.
    //
    while ( ( (cTmpChar = rtSOURCE_ISTREAM.get()) != ';'  ) && 
            (  cTmpChar                           != '\n' ) &&
            rtSOURCE_ISTREAM.good()                          )
    {
      yValue += cTmpChar;
    }
  }

}  // read()


void mpcl::net::cgi::TCookie::
write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const
{

  using std::endl;

  //
  //  Minimum data: name and value of the cookie
  //
  rtTARGET_OSTREAM << "Set-Cookie: " << yName << "=" << yValue;

  //
  //  Optional attributes.
  //
  if ( !tExpires.empty() )
  {
    rtTARGET_OSTREAM << "; expires=" << tExpires.get ("%a, %d-%b-%Y %H:%M:%S GMT");
  }
  if ( !yPath.empty() )
  {
    rtTARGET_OSTREAM << "; path=" << yPath;
  }
  if ( !yDomain.empty() )
  {
    rtTARGET_OSTREAM << "; domain=" << yDomain;
  }
  if ( gSecure )
  {
    rtTARGET_OSTREAM << "; secure";
  }
  rtTARGET_OSTREAM << endl;

}  // write()
