/*
*  Name:      orb.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   CORBA ORB class
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2001-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <mpcl/net/corba/exceptions.hh>
#include <mpcl/net/corba/orb.hh>


//
//  C O N S T R U C T O R S
//

mpcl::net::corba::TOrb::
TOrb (int iPARAMETER_COUNT, const char** ppkcPARAMETERS) :
  gIsColocated        (false)                            ,
  gHasNameService     (false)                            ,
  qtOrb               ()                                 ,
  qtPoa               ()                                 ,
  qtPoaManager        ()                                 ,
  qtRootNamingContext ()
{

  CORBA::Object_var   qtObject;

  //
  //  Obtains object references for the root POA and the COS Naming Service.
  //
  qtOrb        = CORBA::ORB_init (iPARAMETER_COUNT, const_cast<char**> (ppkcPARAMETERS));
  qtObject     = qtOrb->resolve_initial_references ("RootPOA");
  qtPoa        = PortableServer::POA::_narrow (qtObject);
  qtPoaManager = qtPoa->the_POAManager();
  // <FIXME id="NameService" comment="the only exception catched should be CORBA::ORB::InvalidName">
  try
  {
    qtObject            = qtOrb->resolve_initial_references ("NameService");
    qtRootNamingContext = CosNaming::NamingContext::_narrow (qtObject);
    gHasNameService     = true;
  }
  //
  //  Silently ignores COS Naming Service failure.
  //
  catch (const CORBA::ORB::InvalidName&) {}
  catch (const CORBA::SystemException&) {}
  // </FIXME>

}  // TOrb()


CosNaming::NamingContext_var mpcl::net::corba::TOrb::
bindContext ( CosNaming::NamingContext_var& rqtNAMING_CONTEXT ,
              const text::TString&          rkyCONTEXT_NAME   )
{

  CosNaming::NamingContext_var   qtNamingContext;

  if ( !gHasNameService )
  {
    throw TNoNameServiceException ("can not bind naming context", __FILE__, __LINE__);
  }
  else
  {
    CosNaming::Name   tContextName;

    tContextName.length (1);
    tContextName [0].id   = rkyCONTEXT_NAME.c_str();
    tContextName [0].kind = "";
    try
    {
      //
      //  Binds the context to root.
      //
      qtNamingContext = rqtNAMING_CONTEXT->bind_new_context (tContextName);
    }
    catch (const CosNaming::NamingContext::AlreadyBound&)
    {
      //
      //  If the context already exists,  this exception will be raised.  In this
      //  case, just  resolve the  name and assign  qtNamingContext to the object
      //  returned.
      //
      CORBA::Object_var   qtObject = rqtNAMING_CONTEXT->resolve (tContextName);

      qtNamingContext = CosNaming::NamingContext::_narrow (qtObject);
      if( CORBA::is_nil (qtNamingContext) )
      {
        throw TNarrowFailedException ("on naming context", __FILE__, __LINE__);
      }
    }
  }
  return qtNamingContext;

}  // bindContext()
