/*
*  Name:      url_encoder.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   URL encoder class
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cctype>
#include <cstdio>
#include <mpcl/net/url_encoder.hh>
#include <mpcl/text/string.hh>


//
//  S E L E C T O R S
//

mpcl::text::TString mpcl::net::TUrlEncoder::
_encode (const text::TString& rkySOURCE_PLAIN)
{

  using std::isalnum;
  using std::sprintf;

  TString       yEncoded;
  char          acConvertedBuffer[] = "  ";
  const char*   pkcSourcePlain      = rkySOURCE_PLAIN.c_str();
  
  //
  //  Replace hexadecimal escape patterns, by their
  //  ASCII values.
  //
  for (register size_t J = 0, zLength = rkySOURCE_PLAIN.size(); ( J < zLength ) ;++J)
  {
    if ( isalnum (*pkcSourcePlain) )
    {
      yEncoded += *pkcSourcePlain;
    }
    else
    {
      yEncoded += '%';
      sprintf (acConvertedBuffer, "%02x", *pkcSourcePlain);
      yEncoded += acConvertedBuffer;
    }
    ++pkcSourcePlain;
  }
  yEncoded.replaceAll (' ', '+');
  return yEncoded;

}  // _encode()


mpcl::text::TString mpcl::net::TUrlEncoder::
_encode ( const util::collection::TStringToStringMap& rktSOURCE_PARAMETER_MAP ,
          bool                                        gEMPTY_PARAMETERS_TOO   )
{

  using util::collection::TStringToStringMap;

  TString                              yParameter;
  TString                              yCgiParameters;
  TStringToStringMap::const_iterator   ktIter = rktSOURCE_PARAMETER_MAP.begin();
  TStringToStringMap::const_iterator   ktEnd  = rktSOURCE_PARAMETER_MAP.end();

  if ( gEMPTY_PARAMETERS_TOO )
  {
    for (; ( ktIter != ktEnd ) ;++ktIter)
    {
      yParameter     += ktIter->first;
      yParameter     += "=" ;
      yParameter     += _encode (ktIter->second);
      yCgiParameters += yParameter;
      yParameter      = "&";
    }
  }
  else
  {
    for (; ( ktIter != ktEnd ) ;++ktIter)
    {
      if ( !ktIter->second.empty() )
      {
        yParameter     += ktIter->first;
        yParameter     += "=" ;
        yParameter     += _encode (ktIter->second);
        yCgiParameters += yParameter;
        yParameter      = "&";
      }
    }
  }
  return yCgiParameters;

}  // _encode()
