/*
*  Name:         date.cc
*  Author:       Francisco Rodrigo Escobedo Robles
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Base date class
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 2000-2001  Francisco Rodrigo Escobedo Robles <frer@dedaloingenieros.com>
*  Copyright (C) 2002       Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <ctime>
#include <mpcl/text/date.hh>


//
//  C O N S T R U C T O R S
//

mpcl::text::IDate::
IDate (const IDate::EDateType keDATE_TYPE)
  throw (TConstraintException)    :
  tSecondsFromEpoch ((time_t) -1)
{

  using std::time;
  using std::time_t;

  switch (keDATE_TYPE)
  {
    case eEmpty:
    {
      tSecondsFromEpoch = (time_t) -1;
      break;
    }
    case eNow:
    {
      tSecondsFromEpoch = time (NULL);
      break;
    }
    case eTomorrow:
    {
      tSecondsFromEpoch = time (NULL);
      addDays (1);
      break;
    }
    default:
    {
      throw TConstraintException ("invalid date type", __FILE__, __LINE__);
    }
  }

}  // IDate()


void mpcl::text::IDate::set ( const int kiYEAR    ,
                              const int kiMONTH   ,
                              const int kiDAY     ,
                              const int kiHOUR    ,
                              const int kiMINUTES ,
                              const int kiSECONDS )
  throw (TConstraintException)
{

  using std::mktime;
  using std::time_t;
  using std::tm;
  
  struct tm   sDateTm;

  sDateTm.tm_year  = kiYEAR  - 1900;
  sDateTm.tm_mon   = kiMONTH - 1;
  sDateTm.tm_mday  = kiDAY;
  sDateTm.tm_hour  = kiHOUR;
  sDateTm.tm_min   = kiMINUTES;
  sDateTm.tm_sec   = kiSECONDS;
  sDateTm.tm_isdst = 0;
  if ( (time_t) -1 == (tSecondsFromEpoch = mktime (&sDateTm)) )
  {
    throw TConstraintException ("invalid date", __FILE__, __LINE__);
  }

}  // set()
