/*
*  Name:      layout_table.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML table
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <algorithm>
#include <mpcl/text/html/layout_table.hh>
#include <mpcl/text/html/table_cell.hh>
#include <mpcl/text/html/table_row.hh>


//
//  S E L E C T O R S
//

mpcl::text::html::QTTag mpcl::text::html::TLayoutTable::
findTag (const TString& rkyIDENTIFIER) const
{

  TStringToTagMap::const_iterator   I;
  TStringToTagMap::const_iterator   J;
  TStringToTagMap::const_iterator   K;
  bool                                  gHasMoreIdentifiers;
// <DELETE id="smart-pointers">
//   QKTTableCell                      qktCell;
//   QKTContainer                      qktContainer;
//   QKTTableRow                       qktRow;
// </DELETE>
  const TTableCell*                 pktCell;
  const TContainer*                 pktContainer;
  const TTableRow*                  pktRow;
  QTTag                             qtTag;
  TString                               yFirstIdentifierPart;
  TString                               yLastIdentifierPart;
  std::size_t                           zOffset;

  zOffset             = rkyIDENTIFIER.find (kcIdentifierSeparator);
  gHasMoreIdentifiers = ( TString::npos != zOffset );
  if ( !gHasMoreIdentifiers )
  {
    yFirstIdentifierPart = rkyIDENTIFIER;
  }
  else
  {
    yFirstIdentifierPart = rkyIDENTIFIER.substr (0, zOffset);
    yLastIdentifierPart  = rkyIDENTIFIER.substr (zOffset + 1);
  }

  //
  //  Searches in for the item into every cell contained in this row.
  //
  I = tStringToTagMap.begin();
  for (; ( I != tStringToTagMap.end() && !qtTag ) ;++I)
  {
    //
    //  Searches all rows.
    //
// <DELETE id="smart-pointers">
//     DynamicCast (qktRow, I->second);
//     J = qktRow->tStringToTagMap.begin();
//     for (; ( ( J != qktRow->tStringToTagMap.end() ) && !qtTag ) ;++J)
// </DELETE>
    pktRow = I->second.dynamicCast<const TTableRow*>();
    J      = pktRow->tStringToTagMap.begin();
    for (; ( ( J != pktRow->tStringToTagMap.end() ) && !qtTag ) ;++J)
    {
      //
      //  Searches for all items in all cells per row.
      //
// <DELETE id="smart-pointers">
//       DynamicCast (qktCell, J->second);
//       K = qktCell->tStringToTagMap.find (yFirstIdentifierPart);
//       if ( K != qktCell->tStringToTagMap.end() )
// </DELETE>
      pktCell = J->second.dynamicCast<const TTableCell*>();
      K       = pktCell->tStringToTagMap.find (yFirstIdentifierPart);
      if ( K != pktCell->tStringToTagMap.end() )
      {
        if ( !gHasMoreIdentifiers )
        {
          qtTag = K->second;
        }
        else
        {
// <DELETE id="smart-pointers">
//           DynamicCast (qktContainer, K->second);
//           qtTag = qktContainer->findTag (yLastIdentifierPart);
// </DELETE>
          pktContainer = K->second.dynamicCast<const TContainer*>();
          qtTag        = pktContainer->findTag (yLastIdentifierPart);
        }
        break;
      }
    }
  }
  return qtTag;

}  // findTag()


mpcl::text::html::QTTag mpcl::text::html::TLayoutTable::
findOrInsertTag (const TString& rkyIDENTIFIER)
{

  TStringToTagMap::iterator   I;
  TStringToTagMap::iterator   J;
  TStringToTagMap::iterator   K;
  bool                            gHasMoreIdentifiers;
// <DELETE id="smart-pointers">
//   QTTableCell                 qtCell;
//   QTContainer                 qtContainer;
//   QTTableRow                  qtRow;
// </DELETE>
  TTableCell*                 ptCell;
  TContainer*                 ptContainer;
  TTableRow*                  ptRow;
  QTTag                       qtTag;
  TString                         yFirstIdentifierPart;
  TString                         yLastIdentifierPart;
  std::size_t                     zOffset;

  zOffset             = rkyIDENTIFIER.find (kcIdentifierSeparator);
  gHasMoreIdentifiers = ( TString::npos != zOffset );
  if ( !gHasMoreIdentifiers )
  {
    yFirstIdentifierPart = rkyIDENTIFIER;
  }
  else
  {
    yFirstIdentifierPart = rkyIDENTIFIER.substr (0, zOffset);
    yLastIdentifierPart  = rkyIDENTIFIER.substr (zOffset + 1);
  }

  //
  //  Searches in for the item into every cell contained in this row.
  //
  I = tStringToTagMap.begin();
  for (; ( I != tStringToTagMap.end() && !qtTag ) ;++I)
  {
    //
    //  Searches all rows.
    //
// <DELETE id="smart-pointers">
//     DynamicCast (qtRow, I->second);
//     J = qtRow->tStringToTagMap.begin();
//     for (; ( ( J != qtRow->tStringToTagMap.end() ) && !qtTag ) ;++J)
// </DELETE>
    ptRow = I->second.dynamicCast<TTableRow*>();
    J     = ptRow->tStringToTagMap.begin();
    for (; ( ( J != ptRow->tStringToTagMap.end() ) && !qtTag ) ;++J)
    {
      //
      //  Searches for all items in all cells per row.
      //
// <DELETE id="smart-pointers">
//       DynamicCast (qtCell, J->second);
//       K = qtCell->tStringToTagMap.find (yFirstIdentifierPart);
//       if ( K != qtCell->tStringToTagMap.end() )
// </DELETE>
      ptCell = J->second.dynamicCast<TTableCell*>();
      K      = ptCell->tStringToTagMap.find (yFirstIdentifierPart);
      if ( K != ptCell->tStringToTagMap.end() )
      {
        if ( !gHasMoreIdentifiers )
        {
          qtTag = K->second;
        }
        else
        {
// <DELETE id="smart-pointers">
//           DynamicCast (qtContainer, K->second);
//           qtTag = qtContainer->findOrInsertTag (yLastIdentifierPart);
// </DELETE>
          ptContainer = K->second.dynamicCast<TContainer*>();
          qtTag       = ptContainer->findOrInsertTag (yLastIdentifierPart);
        }
        break;
      }
    }
  }
  if ( !qtTag && isDynamiclyLoadable() && !gHasMoreIdentifiers )
  {
    qtTag = new TTag (rkyIDENTIFIER.c_str(), true);
// <DELETE id="smart-pointers">
//     qtRow = new TTableRow();
//     qtRow->insert (new TTableCell (qtTag));
//     insert (qtRow);
// </DELETE>
    ptRow = new TTableRow();
    ptRow->insert (QTTag (new TTableCell (qtTag)));
    insert (QTTag (ptRow));
  }
  if ( !qtTag )
  {
    const char*   pkcFormat  = "in '%s' there is no tag called '%s'";
    TString       yMessage (Format (pkcFormat, identifier().c_str(), rkyIDENTIFIER.c_str()));

    throw TConstraintException (yMessage, __FILE__, __LINE__);
  }
  return qtTag;

}  // findOrInsertTag()
