/*
*  Name:      table.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML table
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <algorithm>
#include <mpcl/text/html/table.hh>
#include <mpcl/text/html/table_row.hh>


//
//  C O N S T R U C T O R S
//

mpcl::text::html::TTable::
TTable ( const char* pkcIDENTIFIER ,
         const char* pkcCAPTION    ,
         const char* pkcALIGN      ) :
  mpcl::text::html::TContainer (pkcIDENTIFIER)
{

  tAttributeMap ["class-identifier"] = "table";
  tAttributeMap.bind ("caption",     pkcCAPTION);
  tAttributeMap.bind ("halign",      pkcALIGN);
  tAttributeMap.bind ("width",       "");
  tAttributeMap.bind ("border",      "");
  tAttributeMap.bind ("cellspacing", "");
  tAttributeMap.bind ("cellpadding", "");
  tAttributeMap.bind ("bgcolor",     "");
  gIsQualified = true;

}  // TTable()


//
//  S E L E C T O R S
//

mpcl::text::html::QTTag mpcl::text::html::TTable::
findOrInsertTag (const TString& rkyIDENTIFIER)
{

  TStringToTagMap::iterator   I;
  bool                        gHasMoreIdentifiers;
// <DELETE id="smart-pointers">
//   QTContainer                 qtContainer;
//   QTTableRow                  qtRow;
// </DELETE>
  TContainer*                 ptContainer;
  QTTag                       qtRow;
  QTTag                       qtTag;
  TString                     yFirstIdentifierPart;
  TString                     yLastIdentifierPart;
  std::size_t                 zOffset;

  zOffset             = rkyIDENTIFIER.find (kcIdentifierSeparator);
  gHasMoreIdentifiers = ( TString::npos != zOffset );
  if ( !gHasMoreIdentifiers )
  {
    yFirstIdentifierPart = rkyIDENTIFIER;
  }
  else
  {
    yFirstIdentifierPart = rkyIDENTIFIER.substr (0, zOffset);
    yLastIdentifierPart  = rkyIDENTIFIER.substr (zOffset + 1);
  }

  //
  //  Searches in for the row called yFirstIdentifierPart.
  //
  I = tStringToTagMap.find (rkyIDENTIFIER);
  if ( I != tStringToTagMap.end() )
  {
    if ( !gHasMoreIdentifiers )
    {
      qtTag = I->second;
    }
    else
    {
// <DELETE id="smart-pointers">
//       DynamicCast (qtContainer, I->second);
//       qtTag = qtContainer->findOrInsertTag (yLastIdentifierPart);
// </DELETE>
      ptContainer = I->second.dynamicCast<TContainer*>();
      qtTag       = ptContainer->findOrInsertTag (yLastIdentifierPart);
    }
  }
  else if ( isDynamiclyLoadable() && !gHasMoreIdentifiers )
  {
    //
    //  There is no such row, so inserts a new one.
    //
    qtRow                           = new TTableRow();
    qtRow->attribute ("identifier") = rkyIDENTIFIER;
    insert (qtRow);
    qtTag = qtRow;
  }
  if ( !qtTag )
  {
    TString   yMessage = Format ("there is no tag called '%s'", rkyIDENTIFIER.c_str());

    throw TConstraintException (yMessage, __FILE__, __LINE__);
  }
  return qtTag;

}  // findOrInsertTag()
