/*
*  Name:         test-cookie.cc
*  Author:       Francisco Rodrigo Escobedo Robles
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      HTTP Cookie testing
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 2000-2001  Francisco Rodrigo Escobedo Robles <frer@dedaloingenieros.com>
*  Copyright (C) 2002       Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/net/cgi/cookie.hh>
#include <mpcl/text/gmt_date.hh>
#include <mpcl/text/string.hh>
#include <mpcl/test.h>
#include <sstream>


int main (void)
{

  using mpcl::net::cgi::TCookie;
  using mpcl::text::TGmtDate;
  using mpcl::text::TString;  
  using std::basic_istringstream;
  using std::basic_ostringstream;

  TEST_INIT ("tests for classes 'TCookie'");

  basic_ostringstream<char>   tTestOutputStream;
  basic_istringstream<char>   tTestInputStream1 ("TestCookieName1=TestCookieValue1; TestCookieName2=TestCookieValue2");
  basic_istringstream<char>   tTestInputStream2 ("TestCookieName3=TestCookieValue3;TestCookieName4=TestCookieValue4;");
  TString                     yCookieOutputValue =
    "Set-Cookie: TestCookieName5=TestCookieValue5; "
    "expires=Fri, 31-Dec-1999 23:00:00 GMT; path=/; "
    "domain=testdomain.com; secure\n";

  TCookie   tCookieTest1 (tTestInputStream1);
  TCookie   tCookieTest2 ("TestCookieName1", "TestCookieValue1");
  TCookie   tCookieTest3 (tTestInputStream1);
  TCookie   tCookieTest4 ("TestCookieName2", "TestCookieValue2");
  TCookie   tCookieTest5 (tTestInputStream2);
  TCookie   tCookieTest6 ("TestCookieName3", "TestCookieValue3");
  TCookie   tCookieTest7 (tTestInputStream2);
  TCookie   tCookieTest8 ("TestCookieName4", "TestCookieValue4");
  TCookie   tCookieTest9 ( "TestCookieName5"              ,
                              "TestCookieValue5"             ,
                              TGmtDate (2000, 1, 1, 0, 0, 0) ,
                              "/"                            ,
                              "testdomain.com"               ,
                              true                           );

  TEST_NUMBERS (true, ( tCookieTest1 == tCookieTest2 ) );
  TEST_NUMBERS (true, ( tCookieTest3 == tCookieTest4 ) );
  TEST_NUMBERS (true, ( tCookieTest5 == tCookieTest6 ) );
  TEST_NUMBERS (true, ( tCookieTest7 == tCookieTest8 ) );

  tTestOutputStream << tCookieTest9;

  TEST_NUMBERS (true, ( tTestOutputStream.str() == yCookieOutputValue ));
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
