/*
*  Name:      test_system.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for operating system
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/system/system.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <vector>


///  Main.
int main (void)
{

  using mpcl::system::TSystem;
  using mpcl::system::tSystem;
  using mpcl::text::TString;
  using mpcl::util::logging::alert;
  using mpcl::util::logging::authpriv;
  using mpcl::util::logging::crit;
  using mpcl::util::logging::cron;
  using mpcl::util::logging::daemon;
  using mpcl::util::logging::debug;
  using mpcl::util::logging::emerg;
  using mpcl::util::logging::err;
  using mpcl::util::logging::header;
  using mpcl::util::logging::info;
  using mpcl::util::logging::kern;
  using mpcl::util::logging::local0;
  using mpcl::util::logging::local1;
  using mpcl::util::logging::local2;
  using mpcl::util::logging::local3;
  using mpcl::util::logging::local4;
  using mpcl::util::logging::local5;
  using mpcl::util::logging::local6;
  using mpcl::util::logging::local7;
  using mpcl::util::logging::lpr;
  using mpcl::util::logging::mail;
  using mpcl::util::logging::news;
  using mpcl::util::logging::notice;
  using mpcl::util::logging::tLog;
  using mpcl::util::logging::user;
  using mpcl::util::logging::uucp;
  using mpcl::util::logging::warning;
  using std::endl;
  using std::vector;

  TEST_INIT ("tests for class 'TSystem'");

  TString                        yPathCollection1 = ":/second/third:last";
  TString                        yPathCollection2 = ":::::last path";
  TString                        yPathCollection3 = ":0:1:2:3:4:";
  TString                        yPathCollection4 = ":0/:1:2/:3:4/:";
  TSystem::TBigEndianBuffer      tBuffer1 ((const unsigned char*) "\01\02\03\04", 4);
  TSystem::TLittleEndianBuffer   tBuffer2 ((const unsigned char*) "\04\03\02\01", 4);
  vector<TString>                tPathVector1 (tSystem.getPathItems (yPathCollection1));
  vector<TString>                tPathVector2 (tSystem.getPathItems (yPathCollection2));
  vector<TString>                tPathVector3 (tSystem.getPathItems (yPathCollection3));
  vector<TString>                tPathVector4 (tSystem.getPathItems (yPathCollection4));

  TEST_NUMBERS (0x01020304, tBuffer1);
  TEST_NUMBERS (0x01020304, tBuffer2);

  TEST_STRINGS (tPathVector1 [0].c_str(), "/second/third/");
  TEST_STRINGS (tPathVector1 [1].c_str(), "last/");
  TEST_NUMBERS (tPathVector1.size(),      2);
  TEST_STRINGS (tPathVector2 [0].c_str(), "last path/");
  TEST_NUMBERS (tPathVector2.size(),      1);
  TEST_STRINGS (tPathVector3 [0].c_str(), "0/");
  TEST_STRINGS (tPathVector3 [1].c_str(), "1/");
  TEST_STRINGS (tPathVector3 [2].c_str(), "2/");
  TEST_STRINGS (tPathVector3 [3].c_str(), "3/");
  TEST_STRINGS (tPathVector3 [4].c_str(), "4/");
  TEST_NUMBERS (tPathVector3.size(),      5);
  TEST_STRINGS (tPathVector4 [0].c_str(), "0/");
  TEST_STRINGS (tPathVector4 [1].c_str(), "1/");
  TEST_STRINGS (tPathVector4 [2].c_str(), "2/");
  TEST_STRINGS (tPathVector4 [3].c_str(), "3/");
  TEST_STRINGS (tPathVector4 [4].c_str(), "4/");
  TEST_NUMBERS (tPathVector4.size(),      5);
  TEST_NUMBERS (true, ( tSystem.getFileSize (__FILE__) > __LINE__ ));

  //
  //  Writes log messages also on 'cerr'.
  //
  if ( !TEST_IN_SILENT_MODE )
  {
    tLog.also (std::cerr);
  }
  
  //
  //  Tests log level (must be checked by the user).
  //
  tLog <<            header << "testing class tLog using default log level"   << endl;
  tLog << emerg   << header << "testing class tLog using 'emerg' log level"   << endl;
  tLog << alert   << header << "testing class tLog using 'alert' log level"   << endl;
  tLog << crit    << header << "testing class tLog using 'crit' log level"    << endl;
  tLog << err     << header << "testing class tLog using 'err' log level"     << endl;
  tLog << warning << header << "testing class tLog using 'warning' log level" << endl;
  tLog << notice  << header << "testing class tLog using 'notice' log level"  << endl;
  tLog << info    << header << "testing class tLog using 'info' log level"    << endl;
  tLog << debug   << header << "testing class tLog using 'debug' log level"   << endl;

  //
  //  Tests log facilities (must be checked by the user).
  //
  tLog << authpriv << header << "testing class tLog using 'authpriv' log facility" << endl;
  tLog << cron     << header << "testing class tLog using 'cron'     log facility" << endl;
  tLog << daemon   << header << "testing class tLog using 'daemon'   log facility" << endl;
  tLog << kern     << header << "testing class tLog using 'kern'     log facility" << endl;
  tLog << local0   << header << "testing class tLog using 'local0'   log facility" << endl;
  tLog << local1   << header << "testing class tLog using 'local1'   log facility" << endl;
  tLog << local2   << header << "testing class tLog using 'local2'   log facility" << endl;
  tLog << local3   << header << "testing class tLog using 'local3'   log facility" << endl;
  tLog << local4   << header << "testing class tLog using 'local4'   log facility" << endl;
  tLog << local5   << header << "testing class tLog using 'local5'   log facility" << endl;
  tLog << local6   << header << "testing class tLog using 'local6'   log facility" << endl;
  tLog << local7   << header << "testing class tLog using 'local7'   log facility" << endl;
  tLog << lpr      << header << "testing class tLog using 'lpr'      log facility" << endl;
  tLog << mail     << header << "testing class tLog using 'mail'     log facility" << endl;
  tLog << news     << header << "testing class tLog using 'news'     log facility" << endl;
  tLog << user     << header << "testing class tLog using 'user'     log facility" << endl;
  tLog << uucp     << header << "testing class tLog using 'uucp'     log facility" << endl;

  //
  //  Tests for other methods (must be checked by the user).
  //
  tLog.debug ("testing method TLog::debug() method");

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
