/*
*  Name:      test-pattern_based_code_generator.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for CODEGEN
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/text/codegen/pattern_based_code_generator.hh>
#include <mpcl/test.h>


///  Main.
int main (void)
{

  using mpcl::text::codegen::TBadInstantiateSyntaxException;
  using mpcl::text::codegen::TPatternBasedCodeGenerator;
  using mpcl::text::TString;
  using mpcl::util::collection::TStringToStringMap;

  TEST_INIT ("tests for class 'TPatternBasedCodeGenerator'");

  TString                      yCorrectInstance11;
  TString                      yCorrectInstance12;
  TString                      yInstance1;
  TString                      yInstance2;
  TString                      yInstance3;
  TString                      yInstance4;
  TString                      yInstance5;
  TPatternBasedCodeGenerator   tCG1;
  TPatternBasedCodeGenerator   tCG2 ("test-pattern_based_code_generator.cdml", ".");
  TStringToStringMap           tStringToStringMap;
  
  TEST_NUMBERS (true,  tCG2.existsClause ("instruction-1", "clause-1"));
  TEST_NUMBERS (true,  tCG2.existsClause ("instruction-1", "clause-2"));
  TEST_NUMBERS (false, tCG2.existsClause ("instruction-3", ""));
  TEST_NUMBERS (false, tCG2.existsClause ("instruction-1", "clause-3"));

  //
  //  Checks for instantiation.
  //
  tStringToStringMap.bind ("param_1", "1-1-value_1");
  tStringToStringMap.bind ("param_2", "1-1-value_2");
  yCorrectInstance11 = "          <1-1-value_1>\n0123456789<1-1-value_2>\n";
  yCorrectInstance12 = "";
  yInstance1 = tCG2 ["instruction-1"]["clause-1"].instantiate (tStringToStringMap);
  yInstance2 = tCG2 ["instruction-1"]["clause-1"].instantiate ( "param_1"     ,
                                                                "1-1-value_1" ,
                                                                "param_2"     ,
                                                                "1-1-value_2" ,
                                                                NULL          );
  try
  {
    yInstance3 = tCG2.codePatternFor ("instruction-1", "clause-2").instantiate ( "param_1"     ,
                                                                                 "1-2-value_1" ,
                                                                                 NULL          );
    TEST_FAIL;
  }
  catch (const TBadInstantiateSyntaxException& rktEXCEPTION)
  {
    //
    //  There is no 'param_1' in clause code.
    //
    TEST_PASS;
  }
  yInstance4 = tCG2.codePatternFor ("instruction-1", "clause-2").instantiate();
  yInstance5 = tCG2 ["instruction-1"]["clause-2"].instantiate();
  TEST_STRINGS (yCorrectInstance11.c_str(), yInstance1.c_str());
  TEST_STRINGS (yCorrectInstance11.c_str(), yInstance2.c_str());
  TEST_STRINGS (yCorrectInstance12.c_str(), yInstance3.c_str());
  TEST_STRINGS (yCorrectInstance12.c_str(), yInstance4.c_str());
  TEST_STRINGS (yCorrectInstance12.c_str(), yInstance5.c_str());

  //
  //  Sets an inexistent path and tries to load
  //  an inexistent CDML document.
  //
  tCG1.setCdmlPath ("/there/is/no/such/path/");
  try
  {
    tCG1.loadFrom ("inexistent file");
    TEST_FAIL;
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    TEST_PASS;
  }
    
  //
  //  Set an existent path and tries to load
  //  an inexistent CDML document.
  //
  tCG1.setCdmlPath ("/");
  try
  {
    tCG1.loadFrom ("inexistent file");
    TEST_FAIL;
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    TEST_PASS;
  }

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
