/*
*  Name:      test-all.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   HTML library test
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <fstream>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/codegen/pattern_based_code_generator.hh>
#include <mpcl/text/html/anchor.hh>
#include <mpcl/text/html/body.hh>
#include <mpcl/text/html/checkbox.hh>
#include <mpcl/text/html/checkbox_group.hh>
#include <mpcl/text/html/form.hh>
#include <mpcl/text/html/horiz_rule.hh>
#include <mpcl/text/html/hidden.hh>
#include <mpcl/text/html/image_submit_group.hh>
#include <mpcl/text/html/input_image.hh>
#include <mpcl/text/html/layout_table.hh>
#include <mpcl/text/html/option.hh>
#include <mpcl/text/html/page.hh>
#include <mpcl/text/html/password.hh>
#include <mpcl/text/html/pattern_page.hh>
#include <mpcl/text/html/radio.hh>
#include <mpcl/text/html/radio_group.hh>
#include <mpcl/text/html/reset.hh>
#include <mpcl/text/html/select.hh>
#include <mpcl/text/html/submit.hh>
#include <mpcl/text/html/table.hh>
#include <mpcl/text/html/table_cell.hh>
#include <mpcl/text/html/table_row.hh>
#include <mpcl/text/html/text.hh>
#include <mpcl/text/html/text_entry.hh>


using mpcl::text::codegen::TPatternBasedCodeGenerator;
using mpcl::text::html::QTTag;
using mpcl::text::html::TAnchor;
using mpcl::text::html::TBody;
using mpcl::text::html::TCheckbox;
using mpcl::text::html::TCheckboxGroup;
using mpcl::text::html::TForm;
using mpcl::text::html::THorizRule;
using mpcl::text::html::THidden;
using mpcl::text::html::TImageSubmitGroup;
using mpcl::text::html::TInputImage;
using mpcl::text::html::TLayoutTable;
using mpcl::text::html::TOption;
using mpcl::text::html::TPage;
using mpcl::text::html::TPatternPage;
using mpcl::text::html::TPassword;
using mpcl::text::html::TRadio;
using mpcl::text::html::TRadioGroup;
using mpcl::text::html::TReset;
using mpcl::text::html::TSelect;
using mpcl::text::html::TSubmit;
using mpcl::text::html::TTable;
using mpcl::text::html::TTableCell;
using mpcl::text::html::TTableRow;
using mpcl::text::html::TTag;
using mpcl::text::html::TText;
using mpcl::text::html::TTextEntry;


//
//  L O C A L   F U N C T I O N S
//

static int _TestPatternPage (void)
{

  using std::cout;

  TAnchor*       ptAnchor1;
  TAnchor*       ptAnchor2;
  TAnchor*       ptAnchor3;
  TPatternPage   tPatternPage ("PatternPage-1", "test-pattern_page.html");

  TEST_INIT ("tests for class 'TPatternPage'");
  ptAnchor1 = new TAnchor ("", "anchor-1", "title", QTTag (new TText ("One")));
  ptAnchor2 = new TAnchor ("", "anchor-2", "title", QTTag (new TText ("Two")));
  ptAnchor3 = new TAnchor ("", "anchor-3", "title", QTTag (new TText ("Three")));
  tPatternPage.insert (QTTag (ptAnchor2));
  tPatternPage.insert (QTTag (ptAnchor3));
  tPatternPage.insert (QTTag (ptAnchor1), tPatternPage.begin());
  TEST_NUMBERS (3,    tPatternPage.size());
  TEST_NUMBERS (true, ( tPatternPage.begin() != tPatternPage.end() ));
  TEST_NUMBERS (true, ( (*tPatternPage.begin())->identifier() == "anchor-1" ));
  if ( !TEST_IN_SILENT_MODE )
  {
    std::cout << tPatternPage;
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestPatternPage()


static int _TestOutput (void)
{

  TEST_INIT ("tests for classes 'html::*'");

  TBody*               ptBody;
  TForm*               ptForm;
  TCheckboxGroup*      ptCheckboxGroup;
  TImageSubmitGroup*   ptImageSubmitGroup;
  TLayoutTable*        ptLayoutTable;
  TRadioGroup*         ptRadioGroup;
  TSelect*             ptSelect;
  TTable*              ptTable;
  TTableRow*           ptTableRow1;
  TTableRow*           ptTableRow2;
  TPage                tPage ("page-1");

  ptForm             = new TForm ("form-1", "cgi-bin/test-cgi", "get");
  ptCheckboxGroup    = new TCheckboxGroup ("cbg1", "vertical");
  ptImageSubmitGroup = new TImageSubmitGroup ("isg1");
  ptLayoutTable      = new TLayoutTable ("layout-table", "Table Sample");
  ptRadioGroup       = new TRadioGroup ("rg1", "value1", "horizontal");
  ptSelect           = new TSelect ("s1", 2, true);
  ptTable            = new TTable ("table", "Table Sample");
  ptTableRow1        = new TTableRow();
  ptTableRow2        = new TTableRow();

  ptForm->insert (QTTag (new THorizRule()));
  ptForm->insert (QTTag (new TTextEntry ("te1", "root", "Nombre: ", 10, 20)));
  ptForm->insert (QTTag (new TPassword ("te2", "Password: ", 20, 30)));

  ptCheckboxGroup->insert (QTTag (new TCheckbox ("cb1", "Checkbox 1", "value1", true)));
  ptCheckboxGroup->insert (QTTag (new TCheckbox ("cb2", "Checkbox 2", "value2", true)));
  ptCheckboxGroup->insert (QTTag (new TCheckbox ("cb3", "Checkbox 3", "value3")));
  ptCheckboxGroup->insert (QTTag (new TCheckbox ("cb4", "Checkbox 4", "value4")));
  ptForm->insert (QTTag (ptCheckboxGroup));

  ptRadioGroup->insert (QTTag (new TRadio ("rb1", "Radio 1", "value1")));
  ptRadioGroup->insert (QTTag (new TRadio ("rb2", "Radio 2", "value2")));
  ptRadioGroup->insert (QTTag (new TRadio ("rb3", "Radio 3", "value3")));
  ptRadioGroup->insert (QTTag (new TRadio ("rb4", "Radio 4", "value4")));
  ptForm->insert (QTTag (ptRadioGroup));

  ptImageSubmitGroup->insert (QTTag (new TInputImage ("ii1", "src")));
  ptImageSubmitGroup->insert (QTTag (new TInputImage ("ii2", "src")));
  ptImageSubmitGroup->insert (QTTag (new TInputImage ("ii3", "src")));
  ptImageSubmitGroup->insert (QTTag (new TInputImage ("ii4", "src")));
  ptForm->insert (QTTag (ptImageSubmitGroup));

  ptSelect->insert (QTTag (new TOption ("o1", "Option 1", "value1", true)));
  ptSelect->insert (QTTag (new TOption ("o2", "Option 2", "value2")));
  ptSelect->insert (QTTag (new TOption ("o3", "Option 3", "value3", true)));
  ptSelect->insert (QTTag (new TOption ("o4", "Option 4", "value4")));
  ptForm->insert (QTTag (ptSelect));

  ptTableRow1->insert (QTTag (new TTableCell (QTTag (new TText ("1-1")))));
  ptTableRow1->insert (QTTag (new TTableCell (QTTag (new TTextEntry ("te1", "1-2", "root", 10, 20)))));
  ptTableRow2->insert (QTTag (new TTableCell (QTTag (new TText ("2-1")))));
  ptTableRow2->insert (QTTag (new TTableCell (QTTag (new TAnchor ("my-ref.html", "Name", "Title", QTTag (new TText ("2-2")))))));
  ptTable->insert (QTTag (ptTableRow1));
  ptTable->insert (QTTag (ptTableRow2));
  ptForm->insert (QTTag (ptTable));

  ptTableRow1 = new TTableRow();
  ptTableRow2 = new TTableRow();
  ptTableRow1->insert (QTTag (new TTableCell (QTTag (new TText ("text-1-1", "1-1")))));
  ptTableRow1->insert (QTTag (new TTableCell (QTTag (new TTextEntry ("te1", "1-2", "root", 10, 20)))));
  ptTableRow2->insert (QTTag (new TTableCell (QTTag (new TText ("text-2-1", "2-1")))));
  ptTableRow2->insert (QTTag (new TTableCell (QTTag (new TAnchor ("my-ref.html", "Name", "Title", QTTag (new TText ("2-2")))))));
  ptLayoutTable->insert (QTTag (ptTableRow1));
  ptLayoutTable->insert (QTTag (ptTableRow2));
  ptForm->insert (QTTag (ptLayoutTable));

  ptForm->insert (QTTag (new THorizRule()));
  ptForm->insert (QTTag (new TSubmit ("s1", "Submit-1")));
  ptForm->insert (QTTag (new TReset ("r1", "Reset-1")));
  ptForm->insert (QTTag (new THidden ("h1", "Hidden-1")));

  ptBody = tPage.tag ("body").dynamicCast<TBody*>();
  ptBody->insert (QTTag (ptForm));
  ptBody->insert (QTTag (new TAnchor ("", "Alt-message", "Title-message", QTTag (new TText ("Ejemplo de enlace")))));
  if ( !TEST_IN_SILENT_MODE )
  {
    std::cout << tPage;
  }

  //
  //  Checks TLayoutTable.
  //
  ptLayoutTable->setDynamiclyLoadable (true);
  TEST_STRINGS ("1-2", tPage ["body"]["form-1"]["layout-table"]["te1"].getValue().c_str());
  TEST_STRINGS ("1-1", tPage ["body"]["form-1"]["layout-table"]["text-1-1"].getValue().c_str());
  TEST_STRINGS ("2-1", tPage ["body"]["form-1"]["layout-table"]["text-2-1"].getValue().c_str());
  TEST_STRINGS ("text-3-1", (tPage.findOrInsertTag ("body/form-1/layout-table/text-3-1"))->identifier().c_str());

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestOutput()


///  Main.
int main (void)
{

  TEST_INIT ("tests for HTML");
  try
  {
    TPatternBasedCodeGenerator   tPbcg1 ("html-2.0.cdml",           "../../../../../data/cdml");
    TPatternBasedCodeGenerator   tPbcg2 ("html-3.2.cdml",           "../../../../../data/cdml");
    TPatternBasedCodeGenerator   tPbcg3 ("html-3.2-netscape.cdml",  "../../../../../data/cdml");
    TPatternBasedCodeGenerator   tPbcg4 ("html-4.01.cdml",          "../../../../../data/cdml");
    TPatternBasedCodeGenerator   tPbcg5 ("html-4.01-netscape.cdml", "../../../../../data/cdml");
    
    TTag::_setCodeGenerator (tPbcg1); TEST_FUNCTION (_TestOutput());
    TTag::_setCodeGenerator (tPbcg2); TEST_FUNCTION (_TestOutput());
    TTag::_setCodeGenerator (tPbcg3); TEST_FUNCTION (_TestOutput());
    TTag::_setCodeGenerator (tPbcg4); TEST_FUNCTION (_TestOutput());
    TEST_FUNCTION (_TestPatternPage());
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL;
  }
  catch (std::exception& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION.what();
    TEST_FAIL;
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
