/*
*  Name:         test-date.cc
*  Author:       Francisco Rodrigo Escobedo Robles
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Base date class
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 2000  Francisco Rodrigo Escobedo Robles <frer@dedaloingenieros.com>
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <mpcl/text/gmt_date.hh>
#include <mpcl/text/local_date.hh>
#include <mpcl/test.h>


///  Main.
int main (void)
{

  using mpcl::text::IDate;
  using mpcl::text::TGmtDate;
  using mpcl::text::TLocalDate;
  using mpcl::TConstraintException;

  bool         gExceptionTest;
  TGmtDate     tEmptyGmtDate  (IDate::eEmpty);
  TGmtDate     tGmtDate1999   (1999, 12, 31, 23, 59, 59);
  TGmtDate     tGmtDate2000   (2000,  1,  1,  0,  0,  0);
  TLocalDate   tLocalDate1999 (1999, 12, 31, 23, 59, 59);
  TLocalDate   tLocalDate2000 (2000,  1,  1,  0,  0,  0);

  TEST_INIT ("tests for classes 'TGmtDate' and 'TLocalDate'");

  //
  //  Supplied comparators.
  //
  TEST_NUMBERS (false, ( tGmtDate1999   == tGmtDate2000   ));
  TEST_NUMBERS (false, ( tGmtDate1999   >  tGmtDate2000   ));
  TEST_NUMBERS (false, ( tGmtDate1999   >= tGmtDate2000   ));
  TEST_NUMBERS (false, ( tLocalDate1999 == tLocalDate2000 ));
  TEST_NUMBERS (false, ( tLocalDate1999 >  tLocalDate2000 ));
  TEST_NUMBERS (false, ( tLocalDate1999 >= tLocalDate2000 ));
  TEST_NUMBERS (true,  ( tGmtDate1999   != tGmtDate2000   ));
  TEST_NUMBERS (true,  ( tGmtDate1999   <  tGmtDate2000   ));
  TEST_NUMBERS (true,  ( tGmtDate1999   <= tGmtDate2000   ));
  TEST_NUMBERS (true,  ( tLocalDate1999 != tLocalDate2000 ));
  TEST_NUMBERS (true,  ( tLocalDate1999 <  tLocalDate2000 ));
  TEST_NUMBERS (true,  ( tLocalDate1999 <= tLocalDate2000 ));

  //
  //  Rollover tests.
  //
  TEST_NUMBERS (true,  ( tGmtDate1999.addSeconds   (1) == tGmtDate2000   ));
  TEST_NUMBERS (true,  ( tLocalDate1999.addSeconds (1) == tLocalDate2000 ))
  TEST_NUMBERS (false, ( tGmtDate1999                  <  tGmtDate2000   ));
  TEST_NUMBERS (false, ( tLocalDate1999                <  tLocalDate2000 ));

  //
  //  Interval tests.
  //
  tGmtDate1999.set (1999, 12, 31, 23, 59, 59);
  tGmtDate2000.set (2000,  1,  1,  0,  0,  0);
  TEST_NUMBERS (true,  tGmtDate1999.in (tGmtDate1999,          tGmtDate1999));
  TEST_NUMBERS (false, tGmtDate1999.in (tGmtDate2000,          tGmtDate2000));
  TEST_NUMBERS (false, tGmtDate1999.in (tGmtDate2000,          tGmtDate1999));
  TEST_NUMBERS (true,  tGmtDate1999.in (TGmtDate (1970, 1, 1), tGmtDate2000));
  TEST_NUMBERS (true,  tGmtDate1999.in (TGmtDate (1970, 1, 1), tGmtDate1999));
  TEST_NUMBERS (false, tGmtDate2000.in (tGmtDate1999,          tGmtDate1999));
  TEST_NUMBERS (true,  tGmtDate2000.in (tGmtDate2000,          tGmtDate2000));
  TEST_NUMBERS (false, tGmtDate2000.in (tGmtDate2000,          tGmtDate1999));
  TEST_NUMBERS (true,  tGmtDate2000.in (TGmtDate (1970, 1, 1), tGmtDate2000));
  TEST_NUMBERS (false, tGmtDate2000.in (TGmtDate (1970, 1, 1), tGmtDate1999));

  //
  //  Tries to manipulate with empty dates.
  //
  try
  {
    TEST_NUMBERS (false, tGmtDate2000.in (tEmptyGmtDate, tEmptyGmtDate));
    TEST_FAIL;
  }
  catch (const TConstraintException& rktEXCEPTION)
  {
    TEST_PASS;
  }
  
  //
  //  Tries to manipulate an empty date.
  //
  try
  {
    TEST_NUMBERS (false, tEmptyGmtDate.in (tGmtDate1999, tGmtDate2000));
    TEST_FAIL;
  }
  catch (const TConstraintException& rktEXCEPTION)
  {
    TEST_PASS;
  }
  
  //
  //  Tries to manipulate an empty date.
  //
  try
  {
    tEmptyGmtDate.addSeconds (1);
    TEST_FAIL;
  }
  catch (const TConstraintException& rktEXCEPTION)
  {
    TEST_PASS;
  }
  try
  {
    //
    //  Tries to manipulate an empty date.
    //
    gExceptionTest = ( tEmptyGmtDate == tGmtDate2000 );
    TEST_FAIL;
  }
  catch (const TConstraintException& rktEXCEPTION)
  {
    TEST_PASS;
  }
  try
  {
    //
    //  Tries to manipulate an empty date.
    //
    gExceptionTest = ( tEmptyGmtDate <  tGmtDate2000 );
    TEST_FAIL;
  }
  catch (const TConstraintException& rktEXCEPTION)
  {
    TEST_PASS;
  }

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
