/*
*  Name:      test-string.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for string
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/text/string.hh>
#include <mpcl/test.h>


using mpcl::text::Format;
using mpcl::text::TString;


//
//  L O C A L   F U N C T I O N S
//

static int _TestStreams (void)
{

  using std::cin;
  using std::cout;
  using std::endl;

  TString   yTest2;
  TString   yTest1 ("Hola Que Tal");

  TEST_INIT ("test for streams member functions");

  if ( !TEST_IN_SILENT_MODE )
  {
    cout << "[" << yTest1 << "]" << endl;

    yTest1 += " estas cara de pan";
    yTest1 += "";
    yTest2 += " estas cara de pan";
    cout << "[" << yTest1 << "]" << endl;
    cout << "[" << yTest2 << "]" << endl;

    cin >> yTest1;
    cin >> yTest2;

    cout << "[" << yTest1 << "]" << endl;
    cout << "[" << yTest2 << "]" << endl;
  }

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestStreams()


static int _TestStringConversionSpeed (void)
{

  TEST_INIT ("benchmarks for conversion functions");

  TEST_START_WATCH;
  for (long int J = 0; ( J < 250000 ) ;++J)
  {
    TEST_NUMBERS (atoi (Format ("%li", J).c_str()), J);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("'Format()' integer conversion");

  TEST_START_WATCH;
  for (long int J = 0; ( J < 250000 ) ;++J)
  {
    TEST_STRINGS (Format ("%.2f", 1.02).c_str(), "1.02");
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("'Format()' float conversion");

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestStringConversionSpeed()


static int _TestFormatSpeed (void)
{

  TString   yTest1;
  char      acBuffer [1024];
  
  TEST_INIT ("benchmarks for formating support functions");

  TEST_START_WATCH;
  for (long int J = 0; ( J < 250000 ) ;++J)
  {
    yTest1  = '\'';
    yTest1 += "Hola";
    yTest1 += '\'';
    TEST_STRINGS ("'Hola'", yTest1.c_str());
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("C++ string handling");

  TEST_START_WATCH;
  for (long int J = 0; ( J < 250000 ) ;++J)
  {
    TEST_STRINGS ("'Hola'", Format ("'%s'", "Hola").c_str());
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("'Format()' string handling");

  TEST_START_WATCH;
  for (long int J = 0; ( J < 200000 ) ;++J)
  {
    sprintf (acBuffer, "<%.2f>", 123.45);
    TEST_STRINGS ("<123.45>", acBuffer);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("'std::sprintf()' benchmark");

  TEST_START_WATCH;
  for (long int J = 0; ( J < 200000 ) ;++J)
  {
    TEST_STRINGS ("<123.45>", Format ("<%.2f>", 123.45).c_str());
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("'Format()' benchmark");

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestFormatSpeed (void)


///  Main.
int main (void)
{

  TEST_INIT ("tests for class 'TString'");

  TString   yTest2;
  TString   yTest3;
  TString   yTest1 ("Hola Que Tal");

  yTest1 += " estas cara de pan";
  yTest1 += "";
  yTest2 += " estas cara de pan";
  TEST_STRINGS (yTest1.c_str(), "Hola Que Tal estas cara de pan");  
  TEST_STRINGS (yTest2.c_str(), " estas cara de pan");  

  yTest1.uppercase();
  yTest2.lowercase();

  TEST_STRINGS (yTest1.c_str(), "HOLA QUE TAL ESTAS CARA DE PAN");
  TEST_STRINGS (yTest2.c_str(), " estas cara de pan");

  yTest1.replaceAll ('A', 'a');
  yTest2.replaceAll ('a', 'b');
  yTest2.replaceAll ("z", "");
  TEST_STRINGS (yTest1.c_str(), "HOLa QUE TaL ESTaS CaRa DE PaN");
  TEST_STRINGS (yTest2.c_str(), " estbs cbrb de pbn");

  TEST_NUMBERS (yTest1.replaceAll ('T', 't'), 2);
  TEST_NUMBERS (yTest2.replaceAll ("a", ""), 0);
  TEST_NUMBERS (yTest2.replaceAll ("z", ""), 0);
  TEST_STRINGS (yTest1.c_str(), "HOLa QUE taL EStaS CaRa DE PaN");
  TEST_STRINGS (yTest2.c_str(), " estbs cbrb de pbn");

  TEST_NUMBERS (1, TString ("ABC").replaceAll ('A', 'B'));
  TEST_NUMBERS (0, TString ("ABC").replaceAll ('Z', 'B'));
  TEST_NUMBERS (1, TString ("ABC").replaceAll ('C', 'Z'));

  TEST_NUMBERS (1, TString ("ABC").replaceAll ("AB", "ZZZZ"));
  TEST_NUMBERS (1, TString ("ABC").replaceAll ("BC", "ZZZZ"));
  TEST_NUMBERS (3, TString ("AAA").replaceAll ("A",  "AA"));

  yTest1 = "ABC";  yTest1.replaceAll ("AB", "ZZZZ");
  yTest2 = "ABC";  yTest2.replaceAll ("BC", "ZZZZ");
  yTest3 = "AAA-"; yTest3.replaceAll ("A",  "AA");
  TEST_STRINGS (yTest1.c_str(), "ZZZZC");
  TEST_STRINGS (yTest2.c_str(), "AZZZZ");
  TEST_STRINGS (yTest3.c_str(), "AAAAAA-");
  
  //
  //  TString::occurrences()
  //  
  TEST_NUMBERS (2, yTest1.occurrences ("ZZ"));
  TEST_NUMBERS (1, yTest1.occurrences ("ZZZZC"));
  TEST_NUMBERS (0, yTest1.occurrences ("ZZZZC5"));
  TEST_NUMBERS (1, yTest2.occurrences ("AZ"));
  TEST_NUMBERS (6, yTest3.occurrences ("A"));
  TEST_NUMBERS (6, yTest3.occurrences ('A'));
  TEST_NUMBERS (1, yTest3.occurrences ("-"));
  TEST_NUMBERS (1, yTest3.occurrences ('-'));

  yTest1 = "ABC";  yTest1.replaceAll ("AB", "");
  yTest2 = "ABC";  yTest2.replaceAll ("BC", "");
  yTest3 = "AAA-"; yTest3.replaceAll ("A",  "");
  TEST_STRINGS (yTest1.c_str(), "C");
  TEST_STRINGS (yTest2.c_str(), "A");
  TEST_STRINGS (yTest3.c_str(), "-");
  
  //
  //  TString::insert()
  //  
  TEST_STRINGS (TString ("ABC").insert (3, "AB").c_str(), "ABCAB");
  TEST_STRINGS (TString ("ABC").insert (0, "BC").c_str(), "BCABC");
  TEST_STRINGS (TString ("AAA-").insert (1, "-").c_str(), "A-AA-");
  
  //
  //  TString::substr()
  //  
  TEST_STRINGS (TString ("ABC").substr (0, 3).c_str(),  "ABC");
  TEST_STRINGS (TString ("ABC").substr (0, 1).c_str(),  "A");
  TEST_STRINGS (TString ("AAA-").substr (3, 1).c_str(), "-");
  
  //
  //  TString::operator OP ()
  //
  TEST_NUMBERS (100220,  atoi (TString ("100220").c_str()));
  TEST_NUMBERS (-100220, atoi (TString ("-100220").c_str()));
  TEST_NUMBERS (false,   atoi (TString ("0000").c_str()));
  TEST_NUMBERS (true,    atoi (TString ("1").c_str()));
  TEST_NUMBERS (false,   atoi (TString ("Tango").c_str()));
  TEST_NUMBERS (false,   atoi (TString ("Cash").c_str()));

  //
  //  TString::leftFill() && TString::rigthFill()
  //
  TEST_NUMBERS (TString ("AAA").assign (3, 'B'), TString ("BBB"));
  TEST_NUMBERS (TString ("AAA").leftFill ('B', 2), TString ("AAA"));
  TEST_NUMBERS (TString ("AAA").leftFill ('B', 4), TString ("BAAA"));
  TEST_NUMBERS (TString ("AAA").leftFill ('B', 10), TString ("BBBBBBBAAA"));
  TEST_NUMBERS (TString ("AAA").rightFill ('B', 3), TString ("AAA"));
  TEST_NUMBERS (TString ("AAA").rightFill ('B', 2), TString ("AAA"));
  TEST_NUMBERS (TString ("AAA").rightFill ('B', 4), TString ("AAAB"));
  TEST_NUMBERS (TString ("AAA").rightFill ('B', 10), TString ("AAABBBBBBB"));

  //
  //  Format()
  //
  yTest1 = Format ("%s, Hola que tal %s como %% estas %s", "Buenos dias", "fali", "tio");
  TEST_NUMBERS (Format   ("%s", "Hola"), TString ("Hola"));
  TEST_NUMBERS (Format   ("%s,%s", "Hola", "Hola"), TString ("Hola,Hola"));
  TEST_NUMBERS (Format   ("%s%%%s", "Hola", "Hola"), TString ("Hola%Hola"));
  TEST_NUMBERS (Format   ("hola"), TString ("hola"));
  TEST_NUMBERS (yTest1,  TString ("Buenos dias, Hola que tal fali como % estas tio"));
  TEST_STRINGS ("75000", Format ("%ld", 75000).c_str());

  //
  //  TString::suppress()
  //
  TEST_STRINGS (TString ("Hola").suppress (' ').c_str(),                  "Hola");
  TEST_STRINGS (TString ("Hola ").suppress ('l').c_str(),                 "Hoa ");
  TEST_STRINGS (TString ("Hola capullo         ").suppress (' ').c_str(), "Holacapullo");
  TEST_STRINGS (TString ("Hola capullo").suppress ('l').c_str(),          "Hoa capuo");
  TEST_STRINGS (TString ("Hola").suppress ('H').c_str(),                  "ola");

  //
  //  TString::suppressLast()
  //
  TEST_STRINGS (TString ("Hola").suppressLast (' ').c_str(),                  "Hola");
  TEST_STRINGS (TString ("Hola ").suppressLast (' ').c_str(),                 "Hola");
  TEST_STRINGS (TString ("Hola capullo         ").suppressLast (' ').c_str(), "Hola capullo");
  TEST_STRINGS (TString (" Hola capullo        ").suppressLast (' ').c_str(), " Hola capullo");
  TEST_STRINGS (TString (" Hola ").suppressLast (' ').c_str(),                " Hola");

  //
  //  TString::suppressFirst()
  //
  TEST_STRINGS (TString ("Hola").suppressFirst (' ').c_str(),                  "Hola");
  TEST_STRINGS (TString (" Hola ").suppressFirst (' ').c_str(),                "Hola ");
  TEST_STRINGS (TString ("        Hola capullo").suppressFirst (' ').c_str(),  "Hola capullo");
  TEST_STRINGS (TString (" Hola capullo        ").suppressFirst (' ').c_str(), "Hola capullo        ");
  TEST_STRINGS (TString (" Hola ").suppressFirst (' ').c_str(),                "Hola ");

  //
  //  Function templates for TString
  //
  yTest1 = "Hello Dolly. How are you?";
  yTest2 = "Hello Dolly. How are you?";
  TEST_NUMBERS (( yTest1.uppercase() == "HELLO DOLLY. HOW ARE YOU?" ), true);
  TEST_NUMBERS (( yTest2.lowercase() == "hello dolly. how are you?" ), true);

  TEST_FUNCTION (_TestStreams());
  TEST_FUNCTION (_TestStringConversionSpeed());
  TEST_FUNCTION (_TestFormatSpeed());
  try
  {
    TString   yTest4 ((const char*) NULL);

    TEST_FAIL;
  }
  catch (const mpcl::TConstraintException&)
  {
    TEST_PASS;
  }

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
