/*
*  Name:      ICasualObserver.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Casual observer interface
*  Date:      $Date: 2004-08-30 02:26:09 +0200 (lun, 30 ago 2004) $
*  Revision:  $Revision: 10010 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for BEHAVIORAL design patterns.
package org.mpcl.designpatterns.behavioral;

import java.util.Observer;


///  Casual observer interface.
public interface ICasualObserver extends Observer
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Registers this observer instance on the observable objects.
  *  @warning This method should be called at  least one time  in this object's
  *           lifespan because the  method \a dispose calls \a stopObserving().
  *  @see #stopObserving()
  */
  public void startObserving();

  ///  Un-registers this observer instance from the observed objects.
  public void stopObserving();

}  // class ICasualObserver
