/*
*  Name:      TTuple.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Tuple class for the model paradigm
*  Date:      $Date: 2003-12-10 20:10:36 +0100 (mié, 10 dic 2003) $
*  Revision:  $Revision: 8592 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for MODEL (from MVC paradigm) design patterns.
package org.mpcl.designpatterns.mvc.model;


///  Base tuple class.
public class TTuple implements Cloneable
{

  ///  True if the tuple is new.
  private boolean   gIsNew;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new tuple.
  *  @post isNew()
  */
  protected TTuple()
  {
    gIsNew = true;
  }
  
  /**
  *  Builds a new tuple.
  *  @post ( isNew() == gIS_NEW )
  */
  protected TTuple (boolean gIS_NEW)
  {
    gIsNew = gIS_NEW;
  }

  /**
  *  Creates and returns a shallow copy of this object.
  *  @return A copy of this object.
  */
  public Object clone()
  {
    Object   tClonedTuple = null;

    try
    {
      tClonedTuple = super.clone();
    }
    catch (final CloneNotSupportedException ktEXCEPTION) {}
    return tClonedTuple;
  }


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Sets the tuple state.  If the tuple is new, then it has not been read from
  *  the model.  If it is not new (old),  then it has been read from the model.
  *  @post ( isNew() == gIS_NEW )
  */
  public void setIsNew (boolean gIS_NEW)
  {
    gIsNew = gIS_NEW;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Tells if the tuple has been newly created, no read from the model.
  *  @return True if the tuple is new.
  */
  public boolean isNew()
  {
    return gIsNew;
  }

}  // class TTuple
