/*
*  Name:      IResourceSupplier.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Interface for resources containing locale data and user interface data
*  Date:      $Date: 2004-08-02 19:44:09 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9840 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.awt.Color;


/**
*  Interface for resources containing locale-specific dialog and user interface
*  data.
*/
public interface IResourceSupplier
{

  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the boolean bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The boolean.
  *  @warning The resource does not correspond to a resource in a ResourceBundle sense.
  */
  public Boolean getBoolean (final String kyRESOURCE_NAME);

  /**
  *  Returns the color bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The color.
  *  @warning The resource does not correspond to a resource in a ResourceBundle sense.
  */
  public Color getColor (final String kyRESOURCE_NAME);

  /**
  *  Returns the short text bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The text.
  */
  public String getShortText (final String kyRESOURCE_NAME);

  /**
  *  Returns the text bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The text.
  */
  public String getText (final String kyRESOURCE_NAME);

  /**
  *  Returns the tool tip text bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The tool tip text.
  */
  public String getToolTipText (final String kyRESOURCE_NAME);

  /**
  *  Returns the mnemonic bound to the resource named \a kyRESOURCE_NAME.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return The mnemonic.
  */
  public char getMnemonic (final String kyRESOURCE_NAME);

  /**
  *  Returns true if the resource named \a kyRESOURCE_NAME has bound a mnemonic.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return True if the resource has a mnemonic.
  */
  public boolean hasMnemonic (final String kyRESOURCE_NAME);

  /**
  *  Returns true if the resource named \a kyRESOURCE_NAME has bound a tool tip
  *  text.
  *  @param kyRESOURCE_NAME Resource name.
  *  @return True if the resource has a tool tip text.
  */
  public boolean hasToolTipText (final String kyRESOURCE_NAME);

}  // class IResourceSupplier
