/*
*  Name:      TModelList.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   List for model instances
*  Date:      $Date: 2004-11-24 06:04:57 +0100 (mié, 24 nov 2004) $
*  Revision:  $Revision: 10489 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.util.ArrayList;
import java.util.List;
import javax.swing.JList;


/**
*  Combo-box for selecting an  instance from  all the instances hold by a model
*  adapter.
*/
public class TModelList extends JList
{

  ///  Model adapter.
  private TAbstractModelToContainerAdapter   tModelToContainerAdapter;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tMODEL_TO_CONTAINER_ADAPTER Model to container adapter instance.
  */
  public TModelList (TAbstractModelToContainerAdapter tMODEL_TO_CONTAINER_ADAPTER)
  {
    super (tMODEL_TO_CONTAINER_ADAPTER);
    tModelToContainerAdapter = tMODEL_TO_CONTAINER_ADAPTER;
  }

  /**
  *  Selects the instance with code \a tCODE  (it is not the item index).  That
  *  code can be null.
  *  @param tCODE Code of the instance.
  */
  public void setSelectedCode (Integer tCODE)
  {
    Integer   tCurrentCode;

    for (int I = 0; ( I < tModelToContainerAdapter.getSize() ) ;++I)
    {
      tCurrentCode = tModelToContainerAdapter.getCodeAt (I);
      if ( ( tCODE == tCurrentCode ) || tCODE.equals (tCurrentCode) )
      {
        setSelectedIndex (I);
        break;
      }
    }
  }

  /**
  *  Selects the instance with code \a iCODE (it is not the item index).
  *  @param iCODE Code of the instance.
  */
  public void setSelectedCode (int iCODE)
  {
    setSelectedCode (new Integer (iCODE));
  }

  /**
  *  Selects  the instances  with code in  \a tCODE_LIST  (it is  not the  item
  *  index). Before doing anything, the selection is cleared.
  *  @param tCODE_LIST Codes of the instances.
  */
  public void setSelectedCodes (List tCODE_LIST)
  {

    int       iModelToContainerAdapterSize;
    Integer   tCurrentCode;
    int[]     aiSelectedIndices;

    clearSelection();
    if ( !tCODE_LIST.isEmpty() )
    {
      iModelToContainerAdapterSize = tModelToContainerAdapter.getSize();
      aiSelectedIndices            = new int [tCODE_LIST.size()];
      for (int I = 0; ( I < tCODE_LIST.size() ) ;++I)
      {
        for (int J = 0; ( J < iModelToContainerAdapterSize ) ;++J)
        {
          tCurrentCode = tModelToContainerAdapter.getCodeAt (J);
          if ( tCurrentCode.intValue() == ((Integer) tCODE_LIST.get (I)).intValue() )
          {
            aiSelectedIndices [I] = J;
            break;
          }
        }
      }
      setSelectedIndices (aiSelectedIndices);
    }

  }  // setSelectedCodes()


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the  current selected  instance  with code \a iCODE (it is not the
  *  item index).
  *  @return The code of the current selected instance.
  */
  public Integer getSelectedCode()
  {
    return tModelToContainerAdapter.getCodeAt (getSelectedIndex());
  }

  /**
  *  Returns the  current selected  instance  with code \a iCODE (it is not the
  *  item index).
  *  @return The code of the current selected instance.
  */
  public List getSelectedCodes()
  {
    int[]     aiSelectedIndex;
    Integer   tCode;
    List      tSelectedIndexList = new ArrayList();

    aiSelectedIndex = getSelectedIndices();
    for (int I = 0; ( I < aiSelectedIndex.length ) ;++I)
    {
      tCode = tModelToContainerAdapter.getCodeAt (aiSelectedIndex [I]);
      tSelectedIndexList.add (tCode);
    }
    return tSelectedIndexList;
  }

}  // class TModelList
