/*
*  Name:      TNotEmptyInputVerifier.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Not empty input validator for 'JTextComponent' derived classes
*  Date:      $Date: 2004-09-07 03:37:15 +0200 (mar, 07 sep 2004) $
*  Revision:  $Revision: 10061 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.text.JTextComponent;


/**
*  Not empty validator class for text fields.
*  @see javax.swing.text.JTextComponent.
*/
public class TNotEmptyInputVerifier extends TAbstractInputVerifier
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param yERROR_MESSAGE Error message for not validating this verifier.
  */
  public TNotEmptyInputVerifier (String yERROR_MESSAGE)
  {
    super (yERROR_MESSAGE);
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Checks whether the JComponent's input is not empty.
  *  @param tCOMPONENT Component to check.
  *  @return True if it holds a not empty string.
  *  @pre ( tCOMPONENT instanceOf JTextField )
  *  @see java.swing.JTextField.
  */
  public boolean verify (JComponent tCOMPONENT)
  {
    JTextComponent   tTextComponent = (JTextComponent) tCOMPONENT;

    return ( tTextComponent.getText().trim().length() > 0 );
  }

}  // class TNotEmptyInputVerifier
