/*
*  Name:      TTreeUtilities.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Tree utilities class
*  Date:      $Date: 2004-10-25 18:34:18 +0200 (lun, 25 oct 2004) $
*  Revision:  $Revision: 10328 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.util.Enumeration;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeNode;


///  Tree utilities class.
public class TTreeUtilities
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Collapses all nodes in the tree path.
  *  @param tTREE             The tree object.
  *  @param tPARENT_TREE_PATH Path that has to be collapseed.
  */
  public static void _collapseAll (JTree tTREE, TreePath tPARENT_TREE_PATH)
  {

    Enumeration   I;
    TreeNode      tChildNode;
    TreePath      tChildPath;
    TreeNode      tLastNode = (TreeNode) tPARENT_TREE_PATH.getLastPathComponent();

    if ( tLastNode.getChildCount() > 0 )
    {
      //
      //  Collapses recursively all child nodes.
      //
      I = tLastNode.children();
      while ( I.hasMoreElements() )
      {
        tChildNode = (TreeNode) I.nextElement();
        tChildPath = tPARENT_TREE_PATH.pathByAddingChild (tChildNode);
        _collapseAll (tTREE, tChildPath);
      }
    }

    //
    //  Collapse must be done bottom-up.
    //
    tTREE.collapsePath (tPARENT_TREE_PATH);

  }  // collapseAll()

  /**
  *  Expands all nodes in the tree path.
  *  @param tTREE             The tree object.
  *  @param tPARENT_TREE_PATH Path that has to be expanded.
  */
  public static void _expandAll (JTree tTREE, TreePath tPARENT_TREE_PATH)
  {

    Enumeration   I;
    TreeNode      tChildNode;
    TreePath      tChildPath;
    TreeNode      tLastNode = (TreeNode) tPARENT_TREE_PATH.getLastPathComponent();

    if ( tLastNode.getChildCount() > 0 )
    {
      //
      //  Expands recursively all child nodes.
      //
      I = tLastNode.children();
      while ( I.hasMoreElements() )
      {
        tChildNode = (TreeNode) I.nextElement();
        tChildPath = tPARENT_TREE_PATH.pathByAddingChild (tChildNode);
        _expandAll (tTREE, tChildPath);
      }
    }

    //
    //  Expansion must be done bottom-up.
    //
    tTREE.expandPath (tPARENT_TREE_PATH);

  }  // expandAll()

}  // class TTreeUtilities
