/*
*  Name:      TShadowBorder.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Border class with shadow in SOUTH-EAST
*  Date:      $Date: 2004-01-02 12:16:25 +0100 (vie, 02 ene 2004) $
*  Revision:  $Revision: 8663 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for BORDER (border support classes).
package org.mpcl.nui.border;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import javax.swing.border.Border;


///  Class for borders with shadows in SOUTH-EAST.
public class TShadowBorder implements Border
{

  ///  Bottom size of the shadow.
  private static int   _iBottomSize;

  ///  Right size of the shadow.
  private static int   _iRightSize;

  ///  Darkest shadow color.
  private static Color   _tShadowColor1;

  ///  Brightest shadow color.
  private static Color   _tShadowColor2;

  ///  Static initialization.
  static
  {
    _iBottomSize   = 2;
    _iRightSize    = 2;
    _tShadowColor1 = new Color (0x30000000, true);
    _tShadowColor2 = new Color (0x10000000, true);
  }


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Returns the insets of the border.
  *  @param tCOMPONENT The component for which this border insets value applies.
  *  @return The insets of the border.
  *  @see javax.swing.border.Border#getBorderInsets()
  */
  public Insets getBorderInsets (Component tCOMPONENT)
  {
    return new Insets (0, 0, _iBottomSize, _iRightSize);
  }

  /**
  *  Returns whether or not the border is opaque.  If the border is opaque,  it
  *  is responsible for filling in it's own background when painting.  But this
  *  border shades the background so it must be painted before.
  *  @return True if the border is opaque.
  *  @see javax.swing.border.Border#isBorderOpaque()
  */
  public  boolean isBorderOpaque()
  {
    return false;
  }

  /**
  *  Paints the border for the specified component with the specified position
  *  and size.
  *  @param tCOMPONENT The component for which this border is being painted.
  *  @param tGRAPHICS  The paint graphics.
  *  @param iX         The x position of the painted border.
  *  @param iY         The y position of the painted border.
  *  @param iWIDTH     The width of the painted border.
  *  @param iHEIGHT    The height of the painted border.
  *  @see javax.swing.border.Border#paintBorder()
  */
  public void paintBorder ( Component tCOMPONENT ,
                            Graphics  tGRAPHICS  ,
                            int       iX         ,
                            int       iY         ,
                            int       iWIDTH     ,
                            int       iHEIGHT    )
  {

    int   iX1;
    int   iX2;
    int   iY1;
    int   iY2;

    //
    //  Draws horizontal lines.
    //
    iX1 = iX + 1;
    iY1 = iY + iHEIGHT - _iBottomSize;
    iX2 = iX + iWIDTH  - _iRightSize - 1;
    tGRAPHICS.setColor (_tShadowColor1);
    tGRAPHICS.drawLine (iX1,     iY1,     iX2,     iY1);
    tGRAPHICS.drawLine (iX1 - 1, iY1 - 1, iX1 - 1, iY1 - 1);
    ++iY1;
    ++iX2;
    tGRAPHICS.setColor (_tShadowColor2);
    tGRAPHICS.drawLine (iX1,     iY1,     iX2,     iY1);
    tGRAPHICS.drawLine (iX2,     iY1 - 1, iX2,     iY1 - 1);
    tGRAPHICS.drawLine (iX1 - 1, iY1 - 1, iX1 - 1, iY1 - 1);

    //
    //  Draws vertical lines.
    //
    iX1 = iX + iWIDTH  - _iRightSize;
    iY1 = iY + 1;
    iY2 = iY + iHEIGHT - _iBottomSize - 1;
    tGRAPHICS.setColor (_tShadowColor1);
    tGRAPHICS.drawLine (iX1,     iY1,     iX1,     iY2);
    tGRAPHICS.drawLine (iX1 - 1, iY1 - 1, iX1 - 1, iY1 - 1);
    ++iX1;
    ++iY2;
    tGRAPHICS.setColor (_tShadowColor2);
    tGRAPHICS.drawLine (iX1,     iY1,     iX1,     iY2);
    tGRAPHICS.drawLine (iX1 - 1, iY1 - 1, iX1 - 1, iY1 - 1);

  }  // paintBorder()

}  // class TShadowBorder
