/*
*  Name:      TAbstractEditorDialog.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base dialog class for editing model instances
*  Date:      $Date: 2004-11-13 03:35:03 +0100 (sáb, 13 nov 2004) $
*  Revision:  $Revision: 10439 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI DIALOG.
package org.mpcl.nui.dialog;

import java.awt.Dialog;
import java.awt.Frame;
import java.util.Observable;
import java.util.Observer;
import org.mpcl.designpatterns.mvc.controller.TAbstractController;
import org.mpcl.nui.IEditor;
import org.mpcl.nui.IResourceSupplier;


///  Base dialog class for editing and observing models.
public abstract class TAbstractEditorDialog
  extends    TAbstractDialog
  implements IEditor, Observer
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.  Creates a non-modal  dialog  without  a title  and
  *  without a specified \a Frame owner.  A shared, hidden frame will be set as
  *  the owner of the dialog.
  *  @param tCONTROLLER Controller instance.
  */
  public TAbstractEditorDialog (TAbstractController tCONTROLLER)
  {
    super (tCONTROLLER);
  }

  /**
  *  Builds a new instance.
  *  @param tDIALOG_OWNER
  *         The \a Dialog object owner of this dialog.
  *  @param kyTITLE_RESOURCE_NAME
  *         Resource name of the title.
  *  @param tCONTROLLER
  *         The controller instance.
  *  @param tRESOURCE_SUPPLIER
  *         Resource supplier for strings.
  *  @param gIS_MODAL
  *         True if the dialog is modal  (it does not permit other interaction
  *         with the application).
  */
  public TAbstractEditorDialog ( Dialog              tDIALOG_OWNER         ,
                                 final String        kyTITLE_RESOURCE_NAME ,
                                 TAbstractController tCONTROLLER           ,
                                 IResourceSupplier   tRESOURCE_SUPPLIER    ,
                                 boolean             gIS_MODAL             )
  {

    super (tDIALOG_OWNER, null, tCONTROLLER, gIS_MODAL);

    setTitle (tRESOURCE_SUPPLIER.getText (kyTITLE_RESOURCE_NAME));
  }

  /**
  *  Builds a new instance.
  *  @param tFRAME_OWNER
  *         The \a Frame object owner of this dialog.
  *  @param kyTITLE_RESOURCE_NAME
  *         Resource name of the title.
  *  @param tCONTROLLER
  *         The controller instance.
  *  @param tRESOURCE_SUPPLIER
  *         Resource supplier for strings.
  *  @param gIS_MODAL
  *         True if the dialog is modal  (it does not permit other interaction
  *         with the application).
  */
  public TAbstractEditorDialog ( Frame               tFRAME_OWNER          ,
                                 final String        kyTITLE_RESOURCE_NAME ,
                                 TAbstractController tCONTROLLER           ,
                                 IResourceSupplier   tRESOURCE_SUPPLIER    ,
                                 boolean             gIS_MODAL             )
  {
    super (tFRAME_OWNER, null, tCONTROLLER, gIS_MODAL);

    setTitle (tRESOURCE_SUPPLIER.getText (kyTITLE_RESOURCE_NAME));
  }

  /**
  *  Updates source view elements from the default model.
  *  @see org.mpcl.nui.IBrowser#load()
  */
  public void load()
  {
    load (null);
  }

  /**
  *  Updates source view elements from the model related to \a tOBSERVABLE.
  *  @param tOBSERVABLE The observed object (can be null).
  *  @see org.mpcl.nui.IBrowser#load (Observable)
  */
  public abstract void load (Observable tOBSERVABLE);

  /**
  *  This  method  is  called  whenever  the  observed  object is  changed.  An
  *  application calls an Observable object's \a notifyObservers method to have
  *  all the object's observers notified of the change.
  *  @param tOBSERVABLE The observable object.
  *  @param tARGUMENT   An argument passed to the \a notifyObservers method.
  *  @see java.util.Observable#notifyObservers
  *  @see java.util.Observer#update
  */
  public void update (Observable tOBSERVABLE, Object tARGUMENT)
  {
    load (null);
    repaint();
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Updates model from source view elements.
  *  @see org.mpcl.nui.IEditor#save()
  */
  public abstract void save();

}  // class TAbstractEditorDialog
