/*
*  Name:      TUserAuthenticationDialog.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Dialog for authenticating an user
*  Date:      $Date: 2004-09-09 01:34:15 +0200 (jue, 09 sep 2004) $
*  Revision:  $Revision: 10088 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI DIALOG.
package org.mpcl.nui.dialog;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import org.mpcl.nui.IAuthenticator;
import org.mpcl.nui.IResourceSupplier;


///  User authentication dialog class.
public class TUserAuthenticationDialog
  extends    TAbstractDialog
  implements ActionListener
{

  ///  True if the user has been authenticated.
  private boolean   gIsAuthentic;

  ///  Button for attempting authentication. 
  private JButton   tAcceptButton;

  ///  User authenticator.
  private IAuthenticator   tAuthenticator;

  ///  User name field.
  private JTextField   tNameTextField;

  ///  User password field.
  private JPasswordField   tPasswordField;

  ///  Resource supplier for messages.
  private IResourceSupplier   tResourceSupplier;

  ///  User name.
  private String   yUserName;

  ///  User password.
  private String   yUserPassword;


  ///  Events string definition class.
  public static class _TEvents
  {
    ///  Event identifier for the 'accept' button.
    public static final String   _kyAccept =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TEvents._kyAccept";

    ///  Event identifier for the 'cancel' button.
    public static final String   _kyCancel =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TEvents._kyCancel";
  }

  ///  Class for resource names.
  public static class _TNames
  {

    ///  Key identifier for the 'accept' button.
    public static final String   _kyAccept =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TNames._kyAccept";

    ///  Key identifier for the 'cancel' button.
    public static final String   _kyCancel =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TNames._kyCancel";

    ///  Key identifier for the 'name' label.
    public static final String   _kyName =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TNames._kyName";

    ///  Key identifier for the 'password' label.
    public static final String   _kyPassword =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TNames._kyPassword";

    ///  Key identifier for the dialog title.
    public static final String   _kyTitle =
      "org.mpcl.nui.dialog.TUserAuthenticationDialog._TNames._kyTitle";

  }  // class _TNames


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tRESOURCE_SUPPLIER Resource supplier for strings.
  *  @param tAUTHENTICATOR     Authenticator.
  */
  public TUserAuthenticationDialog ( IResourceSupplier tRESOURCE_SUPPLIER ,
                                     IAuthenticator    tAUTHENTICATOR     )
  {
    super ((JFrame) null, tRESOURCE_SUPPLIER.getText (_TNames._kyTitle), null, true);
    gIsAuthentic      = false;
    tAuthenticator    = tAUTHENTICATOR;
    tResourceSupplier = tRESOURCE_SUPPLIER;
    yUserName         = new String();
    yUserPassword     = null;
    tMainPanel.add (createDataSheetPanel(), BorderLayout.CENTER);
    tMainPanel.add (createButtonPanel(),    BorderLayout.SOUTH);
  }

  /**
  *  Builds a new instance.
  *  @param tRESOURCE_SUPPLIER Resource bundle for strings.
  *  @param tAUTHENTICATOR     Authenticator.
  *  @param yDEFAULT_USER_NAME Default user name (can be empty).
  */
  public TUserAuthenticationDialog ( IResourceSupplier tRESOURCE_SUPPLIER ,
                                     IAuthenticator    tAUTHENTICATOR     ,
                                     String            yDEFAULT_USER_NAME )
  {
    super ((JFrame) null, tRESOURCE_SUPPLIER.getText (_TNames._kyTitle), null, true);
    gIsAuthentic      = false;
    tAuthenticator    = tAUTHENTICATOR;
    tResourceSupplier = tRESOURCE_SUPPLIER;
    yUserName         = yDEFAULT_USER_NAME;
    yUserPassword     = null;
    tMainPanel.add (createDataSheetPanel(), BorderLayout.CENTER);
    tMainPanel.add (createButtonPanel(),    BorderLayout.SOUTH);
  }

  /**
  *  Invoked when an action occurs.
  *  @param tACTION_EVENT The action event.
  */
  public void actionPerformed (ActionEvent tACTION_EVENT)
  {

    String   yCommand = tACTION_EVENT.getActionCommand();

    if ( yCommand.equals (_TEvents._kyAccept) )
    {
      yUserName     = new String (tNameTextField.getText());
      yUserPassword = new String (tPasswordField.getPassword());
      if ( tAuthenticator.authenticate (yUserName, yUserPassword) )
      {
        gIsAuthentic = true;
        setVisible (false);
      }
      else
      {
        yUserName     = null;
        yUserPassword = null;
        gIsAuthentic  = false;
      }
    }
    else if ( yCommand.equals (_TEvents._kyCancel) )
    {
      yUserName     = null;
      yUserPassword = null;
      gIsAuthentic  = false;
      setVisible (false);
    }

  }  // actionPerformed()

  public void initialize()
  {
    getRootPane().setDefaultButton (tAcceptButton);
    tNameTextField.requestFocus();
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Creates the button panel.
  *  @return A button panel.
  */
  private JPanel createButtonPanel()
  {

    JPanel    tButtonPanel  = new JPanel();
    JButton   tCancelButton = new JButton (tResourceSupplier.getText (_TNames._kyCancel));

    tAcceptButton = new JButton (tResourceSupplier.getText (_TNames._kyAccept));
    tAcceptButton.setActionCommand (_TEvents._kyAccept);
    tCancelButton.setActionCommand (_TEvents._kyCancel);
    tAcceptButton.addActionListener (this);
    tCancelButton.addActionListener (this);
    tAcceptButton.setMnemonic (tResourceSupplier.getMnemonic (_TNames._kyAccept));
    tCancelButton.setMnemonic (tResourceSupplier.getMnemonic (_TNames._kyCancel));
    tButtonPanel.setLayout (new BoxLayout (tButtonPanel, BoxLayout.X_AXIS));
    tButtonPanel.setBorder (BorderFactory.createEmptyBorder (_kiEmptyBorderLength, 0, 0, 0));
    tButtonPanel.add (Box.createHorizontalGlue());
    tButtonPanel.add (tAcceptButton);
    tButtonPanel.add (tCancelButton);
    return tButtonPanel;

  }  // createButtonPanel()

  /**
  *  Creates the data sheet panel.
  *  @return A data sheet panel.
  */
  private JPanel createDataSheetPanel()
  {

    JPanel               tDataSheetPanel;
    GridBagConstraints   tGridBagConstraints;
    GridBagLayout        tGridBagLayout;
    JLabel               tNameLabel;
    JLabel               tPasswordLabel;

    tDataSheetPanel     = new JPanel();
    tGridBagConstraints = new GridBagConstraints();
    tGridBagLayout      = new GridBagLayout();
    tNameLabel          = new JLabel (tResourceSupplier.getText (_TNames._kyName),     SwingConstants.LEFT);
    tPasswordLabel      = new JLabel (tResourceSupplier.getText (_TNames._kyPassword), SwingConstants.LEFT);
    tNameTextField      = new JTextField     (yUserName, 8);
    tPasswordField      = new JPasswordField (8);
    tNameLabel.setDisplayedMnemonic     (tResourceSupplier.getMnemonic (_TNames._kyName));
    tPasswordLabel.setDisplayedMnemonic (tResourceSupplier.getMnemonic (_TNames._kyPassword));
    tNameLabel.setLabelFor     (tNameTextField);
    tPasswordLabel.setLabelFor (tPasswordField);
    tDataSheetPanel.setBorder (createBorder());
    tDataSheetPanel.setLayout (tGridBagLayout);

    //
    //  User name label.
    //
    tGridBagConstraints.fill   = GridBagConstraints.VERTICAL;
    tGridBagConstraints.anchor = GridBagConstraints.EAST;
    tGridBagConstraints.gridx  = 0;
    tGridBagConstraints.gridy  = 0;
    tGridBagConstraints.ipadx  = 10;
    tGridBagConstraints.ipady  = 10;
    tGridBagLayout.setConstraints (tNameLabel, tGridBagConstraints);
    tDataSheetPanel.add (tNameLabel);
    tGridBagConstraints.anchor = GridBagConstraints.WEST;

    //
    //  Name text field. It selects all the text.
    //
    tGridBagConstraints.gridx = 1;
    tGridBagLayout.setConstraints (tNameTextField, tGridBagConstraints);
    tNameTextField.selectAll();
    tDataSheetPanel.add (tNameTextField);

    //
    //  Password label.
    //
    tGridBagConstraints.anchor = GridBagConstraints.EAST;
    tGridBagConstraints.gridx  = 0;
    tGridBagConstraints.gridy  = 1;
    tGridBagLayout.setConstraints (tPasswordLabel, tGridBagConstraints);
    tDataSheetPanel.add (tPasswordLabel);
    tGridBagConstraints.anchor = GridBagConstraints.WEST;

    //
    //  Password text field.
    //
    tGridBagConstraints.gridx = 1;
    tGridBagLayout.setConstraints (tPasswordField, tGridBagConstraints);
    tDataSheetPanel.add (tPasswordField);
    return tDataSheetPanel;

  }  // createDataSheetPanel()


  //
  //  S E L E C T O R S
  //

  /**
  *  Gets user authenticated name.
  *  @return The user name.  If the user has not been authenticated then it
  *          returns NULL.
  */
  public String getName()
  {
    return yUserName;
  }

  /**
  *  Gets user authenticated password.
  *  @return The user password.  If the user has not been authenticated then it
  *          returns NULL.
  */
  public String getPassword()
  {
    return yUserPassword;
  }

  /**
  *  Tells if the user has been authenticated.
  *  @return True if the user has been authenticated.
  */
  public boolean isAuthentic()
  {
    return gIsAuthentic;
  }

}  // class TUserAuthenticationDialog
