/*
*  Name:      TPanelUtilities.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Abstract dialog class
*  Date:      $Date: 2005-04-05 15:08:27 +0200 (mar, 05 abr 2005) $
*  Revision:  $Revision: 11007 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI PANEL.
package org.mpcl.nui.panel;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.JPanel;


///  Panel utilities class.
public class TPanelUtilities
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Adds the panel  \a tTITLE_PANEL to  the panel  \a tTARGET_PANEL  using the
  *  layout  \a tGRID_BAG_LAYOUT  and the constraints \a tGRID_BAG_CONSTRAINTS,
  *  but  making  the title panel  to use the  remaining horizontal space.  The
  *  constraints object is not modified. The position is not changed also.
  *  @param tTITLE_PANEL          The title panel object.
  *  @param tGRID_BAG_CONSTRAINTS The grid-bag constraints object.
  *  @param tGRID_BAG_LAYOUT      The grid-bag layout object.
  *  @param tTARGET_PANEL         The target panel object.
  */
  public static void
  _addTitlePanel ( JPanel             tTITLE_PANEL          ,
                   GridBagConstraints tGRID_BAG_CONSTRAINTS ,
                   GridBagLayout      tGRID_BAG_LAYOUT      ,
                   JPanel             tTARGET_PANEL         )
  {

    int      iOldAnchor    = tGRID_BAG_CONSTRAINTS.anchor;
    int      iOldFill      = tGRID_BAG_CONSTRAINTS.fill;
    int      iOldGridWidth = tGRID_BAG_CONSTRAINTS.gridwidth;
    Insets   tOldInsets    = tGRID_BAG_CONSTRAINTS.insets;

    tGRID_BAG_CONSTRAINTS.anchor    = GridBagConstraints.EAST;
    tGRID_BAG_CONSTRAINTS.fill      = GridBagConstraints.HORIZONTAL;
    tGRID_BAG_CONSTRAINTS.gridwidth = GridBagConstraints.REMAINDER;
    tGRID_BAG_CONSTRAINTS.insets    = new Insets (0, 0, 0, 0);
    tGRID_BAG_LAYOUT.setConstraints (tTITLE_PANEL, tGRID_BAG_CONSTRAINTS);
    tTARGET_PANEL.add (tTITLE_PANEL);
    tGRID_BAG_CONSTRAINTS.anchor    = iOldAnchor;
    tGRID_BAG_CONSTRAINTS.fill      = iOldFill;
    tGRID_BAG_CONSTRAINTS.gridwidth = iOldGridWidth;
    tGRID_BAG_CONSTRAINTS.insets    = tOldInsets;

  }  // _addTitlePanel()

  /**
  *  Creates an empty panel  with the  layout \a tGRID_BAG_LAYOUT  and with the
  *  constraints \a tGRID_BAG_CONSTRAINTS. The constraints are modified to make
  *  the empty panel use  the extra horizontal and vertical space in the parent
  *  component.
  *  @param tGRID_BAG_CONSTRAINTS The grid-bag constraints object.
  *  @param tGRID_BAG_LAYOUT      The grid-bag layout object.
  *  @return The new empty panel.
  *  @warning The constraints object is modified.
  */
  public static JPanel
  _createEmptyPanel (GridBagConstraints tGRID_BAG_CONSTRAINTS, GridBagLayout tGRID_BAG_LAYOUT)
  {
    JPanel   tEmptyPanel;

    tEmptyPanel                      = new JPanel();
    tGRID_BAG_CONSTRAINTS.gridheight = GridBagConstraints.REMAINDER;
    tGRID_BAG_CONSTRAINTS.gridwidth  = GridBagConstraints.REMAINDER;
    tGRID_BAG_CONSTRAINTS.weightx    = 1.0;
    tGRID_BAG_CONSTRAINTS.weighty    = 1.0;
    tGRID_BAG_LAYOUT.setConstraints (tEmptyPanel, tGRID_BAG_CONSTRAINTS);
    tEmptyPanel.setBackground (null);
    return tEmptyPanel;
  }

}  // class TPanelUtilities
