/*
*  Name:      ITableCellDecorator.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Table tree cell renderer that displays different colors depending
*             on row evenness
*  Date:      $Date: 2004-05-26 16:46:14 +0200 (mié, 26 may 2004) $
*  Revision:  $Revision: 9444 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.awt.Component;
import javax.swing.JTable;


/**
*  Table cell decorator interface  that modifies the appearance of a component.
*  It implements the Decorator design pattern.
*/
public interface ITableCellDecorator
{

  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the component used for drawing the cell (displaying or editing).
  *  @param tCOMPONENT   Component to be modified.
  *  @param tTABLE       Table where the cell belongs to.
  *  @param tVALUE       Value contained in the cell.
  *  @param gIS_SELECTED True if the cell is selected.
  *  @param gHAS_FOCUS   True if the cell has the focus.
  *  @param iROW         Row of the cell.
  *  @param iCOLUMN      Column of the cell.
  *  @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent()
  */
  public Component getTableCellComponent ( Component tCOMPONENT   ,
                                           JTable    tTABLE       ,
                                           Object    tVALUE       ,
                                           boolean   gIS_SELECTED ,
                                           boolean   gHAS_FOCUS   ,
                                           int       iROW         ,
                                           int       iCOLUMN      );

}  // interface ITableCellDecorator
