/*
*  Name:      TDateTableCellRenderer.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Table cell renderer for dates
*  Date:      $Date: 2004-11-13 03:33:07 +0100 (sáb, 13 nov 2004) $
*  Revision:  $Revision: 10438 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.text.DateFormat;
import java.util.Date;
import javax.swing.table.DefaultTableCellRenderer;
import org.mpcl.text.TDecimalFormat;


/**
*  Table cell renderer for  \a Date objects.  It uses a \a DateFormat object to
*  convert them to strings.
*  @see java.text.DateFormat
*/
public class TDateTableCellRenderer extends DefaultTableCellRenderer
{

  ///  Date format.
  private DateFormat   tDateFormat;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Sets the String object for the cell being rendered to value.
  *  @param tVALUE The value for this cell; if \a tVALUE is \a null it sets the
  *                the text value to an empty string.
  *  @see javax.swing.table.DefaultTableCellRenderer#setValue()
  */
  protected void setValue (Object tVALUE)
  {
    setText (( tVALUE == null ) ? "" : tDateFormat.format ((Date) tVALUE));
  }


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance. The date format style defaults to short.
  *  @see java.text.DateFormat#SHORT
  *  @see java.text.DateFormat#getDateInstance()
  */
  public TDateTableCellRenderer()
  {
    super();
    tDateFormat = DateFormat.getDateInstance (DateFormat.SHORT);
    setHorizontalAlignment (DefaultTableCellRenderer.RIGHT);
  }

  /**
  *  Builds a new instance. The date format style is set to \a iDATE_FORMAT_STYLE
  *  @param iDATE_FORMAT_STYLE The date format style.
  *  @see java.text.DateFormat#getDateInstance()
  */
  public TDateTableCellRenderer (int iDATE_FORMAT_STYLE)
  {
    super();
    tDateFormat = DateFormat.getDateInstance (iDATE_FORMAT_STYLE);
    setHorizontalAlignment (DefaultTableCellRenderer.RIGHT);
  }

  /**
  *  Builds a new instance. The date format style is set to \a iDATE_FORMAT_STYLE
  *  @param iDATE_FORMAT_STYLE The date format style.
  *  @see java.text.DateFormat#getDateInstance()
  */
  public TDateTableCellRenderer (DateFormat tDATE_FORMAT)
  {
    super();
    tDateFormat = tDATE_FORMAT;
    setHorizontalAlignment (DefaultTableCellRenderer.RIGHT);
  }

}  // class TDateTableCellRenderer
