/*
*  Name:      TDecoratorTableCellRenderer.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Table cell renderer that decorates another table cell renderer
*  Date:      $Date: 2004-10-01 02:01:50 +0200 (vie, 01 oct 2004) $
*  Revision:  $Revision: 10189 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.awt.Component;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;


/**
*  Table cell renderer that  decorates a delegate cell renderer.  It implements
*  the Decorator design pattern.
*/
public class TDecoratorTableCellRenderer implements TableCellRenderer
{

  ///  Delegate table cell renderer.
  private TableCellRenderer   tDelegateTableCellRenderer;

  ///  Table cell decorator.
  private ITableCellDecorator   tTableCellDecorator;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tDELEGATE_TABLE_CELL_RENDERER The delegate table cell renderer.
  *  @param tTABLE_CELL_DECORATOR         Table cell decorator.
  */
  public TDecoratorTableCellRenderer ( TableCellRenderer   tDELEGATE_TABLE_CELL_RENDERER ,
                                       ITableCellDecorator tTABLE_CELL_DECORATOR         )
  {
    tDelegateTableCellRenderer = tDELEGATE_TABLE_CELL_RENDERER;
    tTableCellDecorator        = tTABLE_CELL_DECORATOR;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the  component  used for  drawing  the cell got from the  delegate
  *  table cell renderer, and modified by its decorator.
  *  @param tTABLE       Table where the cell belongs to.
  *  @param tVALUE       Value contained in the cell.
  *  @param gIS_SELECTED True if the cell is selected.
  *  @param gHAS_FOCUS   True if the cell has the focus.
  *  @param iROW         Row of the cell.
  *  @param iCOLUMN      Column of the cell.
  *  @return The \a Component that the renderer uses to draw the value.
  *  @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent()
  *  @see javax.swing.JTable#getBackground()
  */
  public Component getTableCellRendererComponent ( JTable  tTABLE       ,
                                                   Object  tVALUE       ,
                                                   boolean gIS_SELECTED ,
                                                   boolean gHAS_FOCUS   ,
                                                   int     iROW         ,
                                                   int     iCOLUMN      )
  {

    Component   tResultComponent;

    tResultComponent =
      tDelegateTableCellRenderer.getTableCellRendererComponent ( tTABLE       ,
                                                                 tVALUE       ,
                                                                 gIS_SELECTED ,
                                                                 gHAS_FOCUS   ,
                                                                 iROW         ,
                                                                 iCOLUMN      );
    tResultComponent =
      tTableCellDecorator.getTableCellComponent ( tResultComponent ,
                                                  tTABLE           ,
                                                  tVALUE           ,
                                                  gIS_SELECTED     ,
                                                  gHAS_FOCUS       ,
                                                  iROW             ,
                                                  iCOLUMN          );
    return tResultComponent;

  }  // getTableCellRendererComponent()

}  // class TDecoratorTableCellRenderer
