/*
*  Name:        ITreeTableModel.java
*  Author:      Philip Milne
*               Scott Violet
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     Tree-table model.
*  Date:        $Date: 2004-09-13 04:07:32 +0200 (lun, 13 sep 2004) $
*  Revision:    $Revision: 10106 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;


/**
*  ITreeTableModel is the model used by a TTreeTable. It extends TreeModel
*  to add methods for getting inforamtion about the set of columns each 
*  node in the ITreeTableModel may have. Each column, like a column in 
*  a TableModel, has a name and a type associated with it. Each node in 
*  the ITreeTableModel can return a value for each of the columns and 
*  set that value if isCellEditable() returns true. 
*/
public interface ITreeTableModel extends TreeModel
{

  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Sets the value for node \a tNODE, at row \a iROW and column \a iCOLUMN.
  *  @param tVALUE  The new value.
  *  @param tNODE   The tree node.
  *  @param iROW    The index of the row.
  *  @param iCOLUMN The index of the column.
  */
  public void setValueAt (Object tVALUE, Object tNODE, int iROW, int iCOLUMN);


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns  the class of  the cell value for a node in the column \a iCOLUMN.
  *  This is used  by the \a JTable to set up a default renderer and editor for
  *  the column.
  *  @param tNODE   The tree node.
  *  @param iCOLUMN The index of the column.
  *  @return The class of the object value in the model.
  */
  public Class getCellClass (Object tNODE, int iCOLUMN);

  /**
  *  Returns the type for column index \a iCOLUMN.
  *  @param iCOLUMN The index of the column.
  *  @return The common ancestor class of the object values in the model.
  */
  public Class getColumnClass (int iCOLUMN);

  /**
  *  Returns the number of available columns.
  *  @return The number of available columns.
  */
  public int getColumnCount();

  /**
  *  Returns the name for column number \a iCOLUMN.
  *  @param iCOLUMN The index of the column.
  *  @return The column name.
  */
  public String getColumnName (int iCOLUMN);

  /**
  *  Returns  the value  to be displayed  for node \a tNODE,  at  column number
  *  \a iCOLUMN.
  *  @param tNODE   The tree node.
  *  @param iCOLUMN The index of the column.
  *  @return The value \a Object at the specified node.
  */
  public Object getValueAt (Object tNODE, int iCOLUMN);

  /**
  *  Returns true if the parent nodes and the node itself is editable.
  *  @param tTREE_PATH The tree-path object.
  *  @see javax.swing.JTree#isPathEditable()
  */
  public boolean isPathEditable (TreePath tTREE_PATH);

  /**
  *  Indicates whether the value for node \a tNODE, at column number \a iCOLUMN
  *  is editable.
  *  @param tNODE   The tree node.
  *  @param iCOLUMN The index of the column.
  *  @return True if the node is editable
  *  @warning The column 0 should always be editable because it is the way the
  *           inner \a JTree object is notified of all events.
  */
  public boolean isCellEditable (Object tNODE, int iCOLUMN);

}  // interface ITreeTableModel
