/*
*  Name:      TAbstractNumberTranslator.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base class for translating numbers to strings and vice-versa using locale.
*  Date:      $Date: 2004-09-21 18:49:46 +0200 (mar, 21 sep 2004) $
*  Revision:  $Revision: 10150 $
*
*  Copyright (C) 2001-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for text related classes.
package org.mpcl.text;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;


/**
*  Class  to  convert   from   numbers  to  their  string   representation  and
*  vice-versa.
*/
public abstract class TAbstractNumberTranslator implements INumberTranslator
{
  
  //
  //  S E L E C T O R S
  //

  /**
  *  Translates the number in \a yNUMBER to a double value.
  *  @param yNUMBER String with the locale representation of a double value.
  *  @pre yNUMBER represents a double value in the selected locale.
  *  @return A double value corresponding to the number in \a yNUMBER converted
  *          using selected locale.
  */
  public abstract double toDouble (String yNUMBER) throws ParseException;

  /**
  *  Translates the number in \a yNUMBER to a float value.
  *  @param yNUMBER String with the locale representation of a float value.
  *  @pre yNUMBER represents a float value in the selected locale.
  *  @return A float value  corresponding to the number in \a yNUMBER converted
  *          using selected locale.
  */
  public abstract float toFloat (String yNUMBER) throws ParseException;

  /**
  *  Translates the number in \a yNUMBER to a integer value.
  *  @param yNUMBER String with the locale representation of a integer value.
  *  @pre yNUMBER represents a integer value in the selected locale.
  *  @return An  integer  value  corresponding  to  the  number  in  \a yNUMBER
  *          converted using selected locale.
  */
  public abstract int toInt (String yNUMBER) throws ParseException;

  /**
  *  Translates  the  number \a dNUMBER  to its  string  representation  in the
  *  selected locale.
  *  @param dNUMBER Double value.
  *  @return An  string with the  representation  of the double value converted
  *          using selected locale.
  */
  public abstract String toString (double dNUMBER);

  /**
  *  Translates  the  number \a tNUMBER  to its  string  representation  in the
  *  selected locale.
  *  @param tNUMBER Number.
  *  @return An  string with the  representation  of the number converted using
  *          selected locale.
  */
  public abstract String toString (Number tNUMBER);

}  // class TAbstractNumberTranslator
